import asyncio

import zendriver as zd
from zendriver import Tab


async def create_account(page: Tab, name: str, email: str, password: str) -> None:
    # Click on the "Sign up" link
    sign_up_link = next(a for a in await page.select_all("a") if "Sign up" in a.text)
    await sign_up_link.click()
    await asyncio.sleep(0.5)

    # Fill in the sign-up form
    name_input = await page.select("#signupName")
    await name_input.send_keys(name)
    email_input = await page.select("#signupEmail")
    await email_input.send_keys(email)
    password_input = await page.select("#signupPassword")
    await password_input.send_keys(password)

    # Click the "Sign Up" button
    sign_up_button = next(
        button for button in await page.select_all("button") if "Sign Up" in button.text
    )
    await sign_up_button.click()
    await asyncio.sleep(0.5)

    # Click through confirmation dialog
    proceed_to_login = await page.find(text="Proceed to Login")
    await proceed_to_login.click()


async def login(page: Tab, email: str, password: str) -> None:
    # Fill in the login form
    email_input = await page.select("#loginEmail")
    await email_input.send_keys(email)
    password_input = await page.select("#loginPassword")
    await password_input.send_keys(password)

    # Click the "Login" button
    login_button = next(
        button for button in await page.select_all("button") if "Login" in button.text
    )
    await login_button.click()
    await asyncio.sleep(0.5)

    # Verify successful login
    message = await page.select("#message")
    if "Welcome back" in message.text_all:
        print("Login successful")
    else:
        print("Login failed")


async def main() -> None:
    browser = await zd.start()
    page = await browser.get(
        "https://slensky.com/zendriver-examples/login-page.html",
    )
    await asyncio.sleep(0.5)  # Wait for the page to load

    name = "John Doe"
    email = "john.doe@example.com"
    password = "securepassword"

    await create_account(page, name, email, password)
    await login(page, email, password)

    await browser.stop()


if __name__ == "__main__":
    asyncio.run(main())



from unittest.mock import Mock

import pytest
from pytest_mock import MockerFixture

from zendriver.core.browser import Browser


@pytest.fixture
def mock_print(mocker: MockerFixture) -> Mock:
    return mocker.patch("builtins.print")


@pytest.fixture
def mock_start(mocker: MockerFixture, browser: Browser) -> Mock:
    return mocker.patch("zendriver.start", return_value=browser)



import pytest
from pytest_mock import MockerFixture

import zendriver as zd
from tests.conftest import CreateBrowser


async def test_connection_error_raises_exception_and_logs_stderr(
    create_browser: type[CreateBrowser],
    mocker: MockerFixture,
    caplog: pytest.LogCaptureFixture,
):
    mocker.patch(
        "zendriver.core.browser.Browser.test_connection",
        return_value=False,
    )
    with caplog.at_level("INFO"):
        with pytest.raises(Exception):
            async with create_browser(
                browser_connection_max_tries=1, browser_connection_timeout=0.1
            ) as _:
                pass
    assert "Browser stderr" in caplog.text


async def test_get_content_gets_html_content(browser: zd.Browser):
    page = await browser.get("https://example.com")
    content = await page.get_content()
    assert content.lower().startswith("<!doctype html>")


async def test_update_target_sets_target_title(browser: zd.Browser):
    page = await browser.get("https://example.com")
    await page.update_target()
    assert page.target
    assert page.target.title == "Example Domain"



# DO NOT EDIT THIS FILE!
#
# This file is generated from the CDP specification. If you need to make
# changes, edit the generator and regenerate all of the modules.
#
# CDP domain: Debugger

from __future__ import annotations
import enum
import typing
from dataclasses import dataclass
from .util import event_class, T_JSON_DICT

from . import runtime
from deprecated.sphinx import deprecated  # type: ignore


class BreakpointId(str):
    """
    Breakpoint identifier.
    """

    def to_json(self) -> str:
        return self

    @classmethod
    def from_json(cls, json: str) -> BreakpointId:
        return cls(json)

    def __repr__(self):
        return "BreakpointId({})".format(super().__repr__())


class CallFrameId(str):
    """
    Call frame identifier.
    """

    def to_json(self) -> str:
        return self

    @classmethod
    def from_json(cls, json: str) -> CallFrameId:
        return cls(json)

    def __repr__(self):
        return "CallFrameId({})".format(super().__repr__())


@dataclass
class Location:
    """
    Location in the source code.
    """

    #: Script identifier as reported in the ``Debugger.scriptParsed``.
    script_id: runtime.ScriptId

    #: Line number in the script (0-based).
    line_number: int

    #: Column number in the script (0-based).
    column_number: typing.Optional[int] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["scriptId"] = self.script_id.to_json()
        json["lineNumber"] = self.line_number
        if self.column_number is not None:
            json["columnNumber"] = self.column_number
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> Location:
        return cls(
            script_id=runtime.ScriptId.from_json(json["scriptId"]),
            line_number=int(json["lineNumber"]),
            column_number=int(json["columnNumber"])
            if json.get("columnNumber", None) is not None
            else None,
        )


@dataclass
class ScriptPosition:
    """
    Location in the source code.
    """

    line_number: int

    column_number: int

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["lineNumber"] = self.line_number
        json["columnNumber"] = self.column_number
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ScriptPosition:
        return cls(
            line_number=int(json["lineNumber"]),
            column_number=int(json["columnNumber"]),
        )


@dataclass
class LocationRange:
    """
    Location range within one script.
    """

    script_id: runtime.ScriptId

    start: ScriptPosition

    end: ScriptPosition

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["scriptId"] = self.script_id.to_json()
        json["start"] = self.start.to_json()
        json["end"] = self.end.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> LocationRange:
        return cls(
            script_id=runtime.ScriptId.from_json(json["scriptId"]),
            start=ScriptPosition.from_json(json["start"]),
            end=ScriptPosition.from_json(json["end"]),
        )


@dataclass
class CallFrame:
    """
    JavaScript call frame. Array of call frames form the call stack.
    """

    #: Call frame identifier. This identifier is only valid while the virtual machine is paused.
    call_frame_id: CallFrameId

    #: Name of the JavaScript function called on this call frame.
    function_name: str

    #: Location in the source code.
    location: Location

    #: JavaScript script name or url.
    #: Deprecated in favor of using the ``location.scriptId`` to resolve the URL via a previously
    #: sent ``Debugger.scriptParsed`` event.
    url: str

    #: Scope chain for this call frame.
    scope_chain: typing.List[Scope]

    #: ``this`` object for this call frame.
    this: runtime.RemoteObject

    #: Location in the source code.
    function_location: typing.Optional[Location] = None

    #: The value being returned, if the function is at return point.
    return_value: typing.Optional[runtime.RemoteObject] = None

    #: Valid only while the VM is paused and indicates whether this frame
    #: can be restarted or not. Note that a ``true`` value here does not
    #: guarantee that Debugger#restartFrame with this CallFrameId will be
    #: successful, but it is very likely.
    can_be_restarted: typing.Optional[bool] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["callFrameId"] = self.call_frame_id.to_json()
        json["functionName"] = self.function_name
        json["location"] = self.location.to_json()
        json["url"] = self.url
        json["scopeChain"] = [i.to_json() for i in self.scope_chain]
        json["this"] = self.this.to_json()
        if self.function_location is not None:
            json["functionLocation"] = self.function_location.to_json()
        if self.return_value is not None:
            json["returnValue"] = self.return_value.to_json()
        if self.can_be_restarted is not None:
            json["canBeRestarted"] = self.can_be_restarted
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> CallFrame:
        return cls(
            call_frame_id=CallFrameId.from_json(json["callFrameId"]),
            function_name=str(json["functionName"]),
            location=Location.from_json(json["location"]),
            url=str(json["url"]),
            scope_chain=[Scope.from_json(i) for i in json["scopeChain"]],
            this=runtime.RemoteObject.from_json(json["this"]),
            function_location=Location.from_json(json["functionLocation"])
            if json.get("functionLocation", None) is not None
            else None,
            return_value=runtime.RemoteObject.from_json(json["returnValue"])
            if json.get("returnValue", None) is not None
            else None,
            can_be_restarted=bool(json["canBeRestarted"])
            if json.get("canBeRestarted", None) is not None
            else None,
        )


@dataclass
class Scope:
    """
    Scope description.
    """

    #: Scope type.
    type_: str

    #: Object representing the scope. For ``global`` and ``with`` scopes it represents the actual
    #: object; for the rest of the scopes, it is artificial transient object enumerating scope
    #: variables as its properties.
    object_: runtime.RemoteObject

    name: typing.Optional[str] = None

    #: Location in the source code where scope starts
    start_location: typing.Optional[Location] = None

    #: Location in the source code where scope ends
    end_location: typing.Optional[Location] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["type"] = self.type_
        json["object"] = self.object_.to_json()
        if self.name is not None:
            json["name"] = self.name
        if self.start_location is not None:
            json["startLocation"] = self.start_location.to_json()
        if self.end_location is not None:
            json["endLocation"] = self.end_location.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> Scope:
        return cls(
            type_=str(json["type"]),
            object_=runtime.RemoteObject.from_json(json["object"]),
            name=str(json["name"]) if json.get("name", None) is not None else None,
            start_location=Location.from_json(json["startLocation"])
            if json.get("startLocation", None) is not None
            else None,
            end_location=Location.from_json(json["endLocation"])
            if json.get("endLocation", None) is not None
            else None,
        )


@dataclass
class SearchMatch:
    """
    Search match for resource.
    """

    #: Line number in resource content.
    line_number: float

    #: Line with match content.
    line_content: str

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["lineNumber"] = self.line_number
        json["lineContent"] = self.line_content
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> SearchMatch:
        return cls(
            line_number=float(json["lineNumber"]),
            line_content=str(json["lineContent"]),
        )


@dataclass
class BreakLocation:
    #: Script identifier as reported in the ``Debugger.scriptParsed``.
    script_id: runtime.ScriptId

    #: Line number in the script (0-based).
    line_number: int

    #: Column number in the script (0-based).
    column_number: typing.Optional[int] = None

    type_: typing.Optional[str] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["scriptId"] = self.script_id.to_json()
        json["lineNumber"] = self.line_number
        if self.column_number is not None:
            json["columnNumber"] = self.column_number
        if self.type_ is not None:
            json["type"] = self.type_
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> BreakLocation:
        return cls(
            script_id=runtime.ScriptId.from_json(json["scriptId"]),
            line_number=int(json["lineNumber"]),
            column_number=int(json["columnNumber"])
            if json.get("columnNumber", None) is not None
            else None,
            type_=str(json["type"]) if json.get("type", None) is not None else None,
        )


@dataclass
class WasmDisassemblyChunk:
    #: The next chunk of disassembled lines.
    lines: typing.List[str]

    #: The bytecode offsets describing the start of each line.
    bytecode_offsets: typing.List[int]

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["lines"] = [i for i in self.lines]
        json["bytecodeOffsets"] = [i for i in self.bytecode_offsets]
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> WasmDisassemblyChunk:
        return cls(
            lines=[str(i) for i in json["lines"]],
            bytecode_offsets=[int(i) for i in json["bytecodeOffsets"]],
        )


class ScriptLanguage(enum.Enum):
    """
    Enum of possible script languages.
    """

    JAVA_SCRIPT = "JavaScript"
    WEB_ASSEMBLY = "WebAssembly"

    def to_json(self) -> str:
        return self.value

    @classmethod
    def from_json(cls, json: str) -> ScriptLanguage:
        return cls(json)


@dataclass
class DebugSymbols:
    """
    Debug symbols available for a wasm script.
    """

    #: Type of the debug symbols.
    type_: str

    #: URL of the external symbol source.
    external_url: typing.Optional[str] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["type"] = self.type_
        if self.external_url is not None:
            json["externalURL"] = self.external_url
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> DebugSymbols:
        return cls(
            type_=str(json["type"]),
            external_url=str(json["externalURL"])
            if json.get("externalURL", None) is not None
            else None,
        )


@dataclass
class ResolvedBreakpoint:
    #: Breakpoint unique identifier.
    breakpoint_id: BreakpointId

    #: Actual breakpoint location.
    location: Location

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["breakpointId"] = self.breakpoint_id.to_json()
        json["location"] = self.location.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ResolvedBreakpoint:
        return cls(
            breakpoint_id=BreakpointId.from_json(json["breakpointId"]),
            location=Location.from_json(json["location"]),
        )


def continue_to_location(
    location: Location, target_call_frames: typing.Optional[str] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Continues execution until specific location is reached.

    :param location: Location to continue to.
    :param target_call_frames: *(Optional)*
    """
    params: T_JSON_DICT = dict()
    params["location"] = location.to_json()
    if target_call_frames is not None:
        params["targetCallFrames"] = target_call_frames
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.continueToLocation",
        "params": params,
    }
    json = yield cmd_dict


def disable() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Disables debugger for given page.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.disable",
    }
    json = yield cmd_dict


def enable(
    max_scripts_cache_size: typing.Optional[float] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, runtime.UniqueDebuggerId]:
    """
    Enables debugger for the given page. Clients should not assume that the debugging has been
    enabled until the result for this command is received.

    :param max_scripts_cache_size: **(EXPERIMENTAL)** *(Optional)* The maximum size in bytes of collected scripts (not referenced by other heap objects) the debugger can hold. Puts no limit if parameter is omitted.
    :returns: Unique identifier of the debugger.
    """
    params: T_JSON_DICT = dict()
    if max_scripts_cache_size is not None:
        params["maxScriptsCacheSize"] = max_scripts_cache_size
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.enable",
        "params": params,
    }
    json = yield cmd_dict
    return runtime.UniqueDebuggerId.from_json(json["debuggerId"])


def evaluate_on_call_frame(
    call_frame_id: CallFrameId,
    expression: str,
    object_group: typing.Optional[str] = None,
    include_command_line_api: typing.Optional[bool] = None,
    silent: typing.Optional[bool] = None,
    return_by_value: typing.Optional[bool] = None,
    generate_preview: typing.Optional[bool] = None,
    throw_on_side_effect: typing.Optional[bool] = None,
    timeout: typing.Optional[runtime.TimeDelta] = None,
) -> typing.Generator[
    T_JSON_DICT,
    T_JSON_DICT,
    typing.Tuple[runtime.RemoteObject, typing.Optional[runtime.ExceptionDetails]],
]:
    """
    Evaluates expression on a given call frame.

    :param call_frame_id: Call frame identifier to evaluate on.
    :param expression: Expression to evaluate.
    :param object_group: *(Optional)* String object group name to put result into (allows rapid releasing resulting object handles using ```releaseObjectGroup````).
    :param include_command_line_api: *(Optional)* Specifies whether command line API should be available to the evaluated expression, defaults to false.
    :param silent: *(Optional)* In silent mode exceptions thrown during evaluation are not reported and do not pause execution. Overrides ````setPauseOnException``` state.
    :param return_by_value: *(Optional)* Whether the result is expected to be a JSON object that should be sent by value.
    :param generate_preview: **(EXPERIMENTAL)** *(Optional)* Whether preview should be generated for the result.
    :param throw_on_side_effect: *(Optional)* Whether to throw an exception if side effect cannot be ruled out during evaluation.
    :param timeout: **(EXPERIMENTAL)** *(Optional)* Terminate execution after timing out (number of milliseconds).
    :returns: A tuple with the following items:

        0. **result** - Object wrapper for the evaluation result.
        1. **exceptionDetails** - *(Optional)* Exception details.
    """
    params: T_JSON_DICT = dict()
    params["callFrameId"] = call_frame_id.to_json()
    params["expression"] = expression
    if object_group is not None:
        params["objectGroup"] = object_group
    if include_command_line_api is not None:
        params["includeCommandLineAPI"] = include_command_line_api
    if silent is not None:
        params["silent"] = silent
    if return_by_value is not None:
        params["returnByValue"] = return_by_value
    if generate_preview is not None:
        params["generatePreview"] = generate_preview
    if throw_on_side_effect is not None:
        params["throwOnSideEffect"] = throw_on_side_effect
    if timeout is not None:
        params["timeout"] = timeout.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.evaluateOnCallFrame",
        "params": params,
    }
    json = yield cmd_dict
    return (
        runtime.RemoteObject.from_json(json["result"]),
        runtime.ExceptionDetails.from_json(json["exceptionDetails"])
        if json.get("exceptionDetails", None) is not None
        else None,
    )


def get_possible_breakpoints(
    start: Location,
    end: typing.Optional[Location] = None,
    restrict_to_function: typing.Optional[bool] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, typing.List[BreakLocation]]:
    """
    Returns possible locations for breakpoint. scriptId in start and end range locations should be
    the same.

    :param start: Start of range to search possible breakpoint locations in.
    :param end: *(Optional)* End of range to search possible breakpoint locations in (excluding). When not specified, end of scripts is used as end of range.
    :param restrict_to_function: *(Optional)* Only consider locations which are in the same (non-nested) function as start.
    :returns: List of the possible breakpoint locations.
    """
    params: T_JSON_DICT = dict()
    params["start"] = start.to_json()
    if end is not None:
        params["end"] = end.to_json()
    if restrict_to_function is not None:
        params["restrictToFunction"] = restrict_to_function
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.getPossibleBreakpoints",
        "params": params,
    }
    json = yield cmd_dict
    return [BreakLocation.from_json(i) for i in json["locations"]]


def get_script_source(
    script_id: runtime.ScriptId,
) -> typing.Generator[
    T_JSON_DICT, T_JSON_DICT, typing.Tuple[str, typing.Optional[str]]
]:
    """
    Returns source for the script with given id.

    :param script_id: Id of the script to get source for.
    :returns: A tuple with the following items:

        0. **scriptSource** - Script source (empty in case of Wasm bytecode).
        1. **bytecode** - *(Optional)* Wasm bytecode. (Encoded as a base64 string when passed over JSON)
    """
    params: T_JSON_DICT = dict()
    params["scriptId"] = script_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.getScriptSource",
        "params": params,
    }
    json = yield cmd_dict
    return (
        str(json["scriptSource"]),
        str(json["bytecode"]) if json.get("bytecode", None) is not None else None,
    )


def disassemble_wasm_module(
    script_id: runtime.ScriptId,
) -> typing.Generator[
    T_JSON_DICT,
    T_JSON_DICT,
    typing.Tuple[typing.Optional[str], int, typing.List[int], WasmDisassemblyChunk],
]:
    """


    **EXPERIMENTAL**

    :param script_id: Id of the script to disassemble
    :returns: A tuple with the following items:

        0. **streamId** - *(Optional)* For large modules, return a stream from which additional chunks of disassembly can be read successively.
        1. **totalNumberOfLines** - The total number of lines in the disassembly text.
        2. **functionBodyOffsets** - The offsets of all function bodies, in the format [start1, end1, start2, end2, ...] where all ends are exclusive.
        3. **chunk** - The first chunk of disassembly.
    """
    params: T_JSON_DICT = dict()
    params["scriptId"] = script_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.disassembleWasmModule",
        "params": params,
    }
    json = yield cmd_dict
    return (
        str(json["streamId"]) if json.get("streamId", None) is not None else None,
        int(json["totalNumberOfLines"]),
        [int(i) for i in json["functionBodyOffsets"]],
        WasmDisassemblyChunk.from_json(json["chunk"]),
    )


def next_wasm_disassembly_chunk(
    stream_id: str,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, WasmDisassemblyChunk]:
    """
    Disassemble the next chunk of lines for the module corresponding to the
    stream. If disassembly is complete, this API will invalidate the streamId
    and return an empty chunk. Any subsequent calls for the now invalid stream
    will return errors.

    **EXPERIMENTAL**

    :param stream_id:
    :returns: The next chunk of disassembly.
    """
    params: T_JSON_DICT = dict()
    params["streamId"] = stream_id
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.nextWasmDisassemblyChunk",
        "params": params,
    }
    json = yield cmd_dict
    return WasmDisassemblyChunk.from_json(json["chunk"])


@deprecated(version="1.3")
def get_wasm_bytecode(
    script_id: runtime.ScriptId,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, str]:
    """
    This command is deprecated. Use getScriptSource instead.

    .. deprecated:: 1.3

    :param script_id: Id of the Wasm script to get source for.
    :returns: Script source. (Encoded as a base64 string when passed over JSON)
    """
    params: T_JSON_DICT = dict()
    params["scriptId"] = script_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.getWasmBytecode",
        "params": params,
    }
    json = yield cmd_dict
    return str(json["bytecode"])


def get_stack_trace(
    stack_trace_id: runtime.StackTraceId,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, runtime.StackTrace]:
    """
    Returns stack trace with given ``stackTraceId``.

    **EXPERIMENTAL**

    :param stack_trace_id:
    :returns:
    """
    params: T_JSON_DICT = dict()
    params["stackTraceId"] = stack_trace_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.getStackTrace",
        "params": params,
    }
    json = yield cmd_dict
    return runtime.StackTrace.from_json(json["stackTrace"])


def pause() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Stops on the next JavaScript statement.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.pause",
    }
    json = yield cmd_dict


@deprecated(version="1.3")
def pause_on_async_call(
    parent_stack_trace_id: runtime.StackTraceId,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """


    .. deprecated:: 1.3

    **EXPERIMENTAL**

    :param parent_stack_trace_id: Debugger will pause when async call with given stack trace is started.
    """
    params: T_JSON_DICT = dict()
    params["parentStackTraceId"] = parent_stack_trace_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.pauseOnAsyncCall",
        "params": params,
    }
    json = yield cmd_dict


def remove_breakpoint(
    breakpoint_id: BreakpointId,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Removes JavaScript breakpoint.

    :param breakpoint_id:
    """
    params: T_JSON_DICT = dict()
    params["breakpointId"] = breakpoint_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.removeBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


def restart_frame(
    call_frame_id: CallFrameId, mode: typing.Optional[str] = None
) -> typing.Generator[
    T_JSON_DICT,
    T_JSON_DICT,
    typing.Tuple[
        typing.List[CallFrame],
        typing.Optional[runtime.StackTrace],
        typing.Optional[runtime.StackTraceId],
    ],
]:
    """
    Restarts particular call frame from the beginning. The old, deprecated
    behavior of ``restartFrame`` is to stay paused and allow further CDP commands
    after a restart was scheduled. This can cause problems with restarting, so
    we now continue execution immediatly after it has been scheduled until we
    reach the beginning of the restarted frame.

    To stay back-wards compatible, ``restartFrame`` now expects a ``mode``
    parameter to be present. If the ``mode`` parameter is missing, ``restartFrame``
    errors out.

    The various return values are deprecated and ``callFrames`` is always empty.
    Use the call frames from the ``Debugger#paused`` events instead, that fires
    once V8 pauses at the beginning of the restarted function.

    :param call_frame_id: Call frame identifier to evaluate on.
    :param mode: **(EXPERIMENTAL)** *(Optional)* The ```mode```` parameter must be present and set to 'StepInto', otherwise ````restartFrame``` will error out.
    :returns: A tuple with the following items:

        0. **callFrames** - New stack trace.
        1. **asyncStackTrace** - *(Optional)* Async stack trace, if any.
        2. **asyncStackTraceId** - *(Optional)* Async stack trace, if any.
    """
    params: T_JSON_DICT = dict()
    params["callFrameId"] = call_frame_id.to_json()
    if mode is not None:
        params["mode"] = mode
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.restartFrame",
        "params": params,
    }
    json = yield cmd_dict
    return (
        [CallFrame.from_json(i) for i in json["callFrames"]],
        runtime.StackTrace.from_json(json["asyncStackTrace"])
        if json.get("asyncStackTrace", None) is not None
        else None,
        runtime.StackTraceId.from_json(json["asyncStackTraceId"])
        if json.get("asyncStackTraceId", None) is not None
        else None,
    )


def resume(
    terminate_on_resume: typing.Optional[bool] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Resumes JavaScript execution.

    :param terminate_on_resume: *(Optional)* Set to true to terminate execution upon resuming execution. In contrast to Runtime.terminateExecution, this will allows to execute further JavaScript (i.e. via evaluation) until execution of the paused code is actually resumed, at which point termination is triggered. If execution is currently not paused, this parameter has no effect.
    """
    params: T_JSON_DICT = dict()
    if terminate_on_resume is not None:
        params["terminateOnResume"] = terminate_on_resume
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.resume",
        "params": params,
    }
    json = yield cmd_dict


def search_in_content(
    script_id: runtime.ScriptId,
    query: str,
    case_sensitive: typing.Optional[bool] = None,
    is_regex: typing.Optional[bool] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, typing.List[SearchMatch]]:
    """
    Searches for given string in script content.

    :param script_id: Id of the script to search in.
    :param query: String to search for.
    :param case_sensitive: *(Optional)* If true, search is case sensitive.
    :param is_regex: *(Optional)* If true, treats string parameter as regex.
    :returns: List of search matches.
    """
    params: T_JSON_DICT = dict()
    params["scriptId"] = script_id.to_json()
    params["query"] = query
    if case_sensitive is not None:
        params["caseSensitive"] = case_sensitive
    if is_regex is not None:
        params["isRegex"] = is_regex
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.searchInContent",
        "params": params,
    }
    json = yield cmd_dict
    return [SearchMatch.from_json(i) for i in json["result"]]


def set_async_call_stack_depth(
    max_depth: int,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Enables or disables async call stacks tracking.

    :param max_depth: Maximum depth of async call stacks. Setting to ```0``` will effectively disable collecting async call stacks (default).
    """
    params: T_JSON_DICT = dict()
    params["maxDepth"] = max_depth
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setAsyncCallStackDepth",
        "params": params,
    }
    json = yield cmd_dict


def set_blackbox_execution_contexts(
    unique_ids: typing.List[str],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Replace previous blackbox execution contexts with passed ones. Forces backend to skip
    stepping/pausing in scripts in these execution contexts. VM will try to leave blackboxed script by
    performing 'step in' several times, finally resorting to 'step out' if unsuccessful.

    **EXPERIMENTAL**

    :param unique_ids: Array of execution context unique ids for the debugger to ignore.
    """
    params: T_JSON_DICT = dict()
    params["uniqueIds"] = [i for i in unique_ids]
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBlackboxExecutionContexts",
        "params": params,
    }
    json = yield cmd_dict


def set_blackbox_patterns(
    patterns: typing.List[str], skip_anonymous: typing.Optional[bool] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Replace previous blackbox patterns with passed ones. Forces backend to skip stepping/pausing in
    scripts with url matching one of the patterns. VM will try to leave blackboxed script by
    performing 'step in' several times, finally resorting to 'step out' if unsuccessful.

    **EXPERIMENTAL**

    :param patterns: Array of regexps that will be used to check script url for blackbox state.
    :param skip_anonymous: *(Optional)* If true, also ignore scripts with no source url.
    """
    params: T_JSON_DICT = dict()
    params["patterns"] = [i for i in patterns]
    if skip_anonymous is not None:
        params["skipAnonymous"] = skip_anonymous
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBlackboxPatterns",
        "params": params,
    }
    json = yield cmd_dict


def set_blackboxed_ranges(
    script_id: runtime.ScriptId, positions: typing.List[ScriptPosition]
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Makes backend skip steps in the script in blackboxed ranges. VM will try leave blacklisted
    scripts by performing 'step in' several times, finally resorting to 'step out' if unsuccessful.
    Positions array contains positions where blackbox state is changed. First interval isn't
    blackboxed. Array should be sorted.

    **EXPERIMENTAL**

    :param script_id: Id of the script.
    :param positions:
    """
    params: T_JSON_DICT = dict()
    params["scriptId"] = script_id.to_json()
    params["positions"] = [i.to_json() for i in positions]
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBlackboxedRanges",
        "params": params,
    }
    json = yield cmd_dict


def set_breakpoint(
    location: Location, condition: typing.Optional[str] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, typing.Tuple[BreakpointId, Location]]:
    """
    Sets JavaScript breakpoint at a given location.

    :param location: Location to set breakpoint in.
    :param condition: *(Optional)* Expression to use as a breakpoint condition. When specified, debugger will only stop on the breakpoint if this expression evaluates to true.
    :returns: A tuple with the following items:

        0. **breakpointId** - Id of the created breakpoint for further reference.
        1. **actualLocation** - Location this breakpoint resolved into.
    """
    params: T_JSON_DICT = dict()
    params["location"] = location.to_json()
    if condition is not None:
        params["condition"] = condition
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBreakpoint",
        "params": params,
    }
    json = yield cmd_dict
    return (
        BreakpointId.from_json(json["breakpointId"]),
        Location.from_json(json["actualLocation"]),
    )


def set_instrumentation_breakpoint(
    instrumentation: str,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, BreakpointId]:
    """
    Sets instrumentation breakpoint.

    :param instrumentation: Instrumentation name.
    :returns: Id of the created breakpoint for further reference.
    """
    params: T_JSON_DICT = dict()
    params["instrumentation"] = instrumentation
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setInstrumentationBreakpoint",
        "params": params,
    }
    json = yield cmd_dict
    return BreakpointId.from_json(json["breakpointId"])


def set_breakpoint_by_url(
    line_number: int,
    url: typing.Optional[str] = None,
    url_regex: typing.Optional[str] = None,
    script_hash: typing.Optional[str] = None,
    column_number: typing.Optional[int] = None,
    condition: typing.Optional[str] = None,
) -> typing.Generator[
    T_JSON_DICT, T_JSON_DICT, typing.Tuple[BreakpointId, typing.List[Location]]
]:
    """
    Sets JavaScript breakpoint at given location specified either by URL or URL regex. Once this
    command is issued, all existing parsed scripts will have breakpoints resolved and returned in
    ``locations`` property. Further matching script parsing will result in subsequent
    ``breakpointResolved`` events issued. This logical breakpoint will survive page reloads.

    :param line_number: Line number to set breakpoint at.
    :param url: *(Optional)* URL of the resources to set breakpoint on.
    :param url_regex: *(Optional)* Regex pattern for the URLs of the resources to set breakpoints on. Either ```url```` or ````urlRegex``` must be specified.
    :param script_hash: *(Optional)* Script hash of the resources to set breakpoint on.
    :param column_number: *(Optional)* Offset in the line to set breakpoint at.
    :param condition: *(Optional)* Expression to use as a breakpoint condition. When specified, debugger will only stop on the breakpoint if this expression evaluates to true.
    :returns: A tuple with the following items:

        0. **breakpointId** - Id of the created breakpoint for further reference.
        1. **locations** - List of the locations this breakpoint resolved into upon addition.
    """
    params: T_JSON_DICT = dict()
    params["lineNumber"] = line_number
    if url is not None:
        params["url"] = url
    if url_regex is not None:
        params["urlRegex"] = url_regex
    if script_hash is not None:
        params["scriptHash"] = script_hash
    if column_number is not None:
        params["columnNumber"] = column_number
    if condition is not None:
        params["condition"] = condition
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBreakpointByUrl",
        "params": params,
    }
    json = yield cmd_dict
    return (
        BreakpointId.from_json(json["breakpointId"]),
        [Location.from_json(i) for i in json["locations"]],
    )


def set_breakpoint_on_function_call(
    object_id: runtime.RemoteObjectId, condition: typing.Optional[str] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, BreakpointId]:
    """
    Sets JavaScript breakpoint before each call to the given function.
    If another function was created from the same source as a given one,
    calling it will also trigger the breakpoint.

    **EXPERIMENTAL**

    :param object_id: Function object id.
    :param condition: *(Optional)* Expression to use as a breakpoint condition. When specified, debugger will stop on the breakpoint if this expression evaluates to true.
    :returns: Id of the created breakpoint for further reference.
    """
    params: T_JSON_DICT = dict()
    params["objectId"] = object_id.to_json()
    if condition is not None:
        params["condition"] = condition
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBreakpointOnFunctionCall",
        "params": params,
    }
    json = yield cmd_dict
    return BreakpointId.from_json(json["breakpointId"])


def set_breakpoints_active(
    active: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Activates / deactivates all breakpoints on the page.

    :param active: New value for breakpoints active state.
    """
    params: T_JSON_DICT = dict()
    params["active"] = active
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setBreakpointsActive",
        "params": params,
    }
    json = yield cmd_dict


def set_pause_on_exceptions(
    state: str,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Defines pause on exceptions state. Can be set to stop on all exceptions, uncaught exceptions,
    or caught exceptions, no exceptions. Initial pause on exceptions state is ``none``.

    :param state: Pause on exceptions mode.
    """
    params: T_JSON_DICT = dict()
    params["state"] = state
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setPauseOnExceptions",
        "params": params,
    }
    json = yield cmd_dict


def set_return_value(
    new_value: runtime.CallArgument,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Changes return value in top frame. Available only at return break position.

    **EXPERIMENTAL**

    :param new_value: New return value.
    """
    params: T_JSON_DICT = dict()
    params["newValue"] = new_value.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setReturnValue",
        "params": params,
    }
    json = yield cmd_dict


def set_script_source(
    script_id: runtime.ScriptId,
    script_source: str,
    dry_run: typing.Optional[bool] = None,
    allow_top_frame_editing: typing.Optional[bool] = None,
) -> typing.Generator[
    T_JSON_DICT,
    T_JSON_DICT,
    typing.Tuple[
        typing.Optional[typing.List[CallFrame]],
        typing.Optional[bool],
        typing.Optional[runtime.StackTrace],
        typing.Optional[runtime.StackTraceId],
        str,
        typing.Optional[runtime.ExceptionDetails],
    ],
]:
    """
    Edits JavaScript source live.

    In general, functions that are currently on the stack can not be edited with
    a single exception: If the edited function is the top-most stack frame and
    that is the only activation of that function on the stack. In this case
    the live edit will be successful and a ``Debugger.restartFrame`` for the
    top-most function is automatically triggered.

    :param script_id: Id of the script to edit.
    :param script_source: New content of the script.
    :param dry_run: *(Optional)* If true the change will not actually be applied. Dry run may be used to get result description without actually modifying the code.
    :param allow_top_frame_editing: **(EXPERIMENTAL)** *(Optional)* If true, then ```scriptSource```` is allowed to change the function on top of the stack as long as the top-most stack frame is the only activation of that function.
    :returns: A tuple with the following items:

        0. **callFrames** - *(Optional)* New stack trace in case editing has happened while VM was stopped.
        1. **stackChanged** - *(Optional)* Whether current call stack  was modified after applying the changes.
        2. **asyncStackTrace** - *(Optional)* Async stack trace, if any.
        3. **asyncStackTraceId** - *(Optional)* Async stack trace, if any.
        4. **status** - Whether the operation was successful or not. Only `` Ok`` denotes a successful live edit while the other enum variants denote why the live edit failed.
        5. **exceptionDetails** - *(Optional)* Exception details if any. Only present when `` status`` is `` CompileError`.
    """
    params: T_JSON_DICT = dict()
    params["scriptId"] = script_id.to_json()
    params["scriptSource"] = script_source
    if dry_run is not None:
        params["dryRun"] = dry_run
    if allow_top_frame_editing is not None:
        params["allowTopFrameEditing"] = allow_top_frame_editing
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setScriptSource",
        "params": params,
    }
    json = yield cmd_dict
    return (
        [CallFrame.from_json(i) for i in json["callFrames"]]
        if json.get("callFrames", None) is not None
        else None,
        bool(json["stackChanged"])
        if json.get("stackChanged", None) is not None
        else None,
        runtime.StackTrace.from_json(json["asyncStackTrace"])
        if json.get("asyncStackTrace", None) is not None
        else None,
        runtime.StackTraceId.from_json(json["asyncStackTraceId"])
        if json.get("asyncStackTraceId", None) is not None
        else None,
        str(json["status"]),
        runtime.ExceptionDetails.from_json(json["exceptionDetails"])
        if json.get("exceptionDetails", None) is not None
        else None,
    )


def set_skip_all_pauses(skip: bool) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Makes page not interrupt on any pauses (breakpoint, exception, dom exception etc).

    :param skip: New value for skip pauses state.
    """
    params: T_JSON_DICT = dict()
    params["skip"] = skip
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setSkipAllPauses",
        "params": params,
    }
    json = yield cmd_dict


def set_variable_value(
    scope_number: int,
    variable_name: str,
    new_value: runtime.CallArgument,
    call_frame_id: CallFrameId,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Changes value of variable in a callframe. Object-based scopes are not supported and must be
    mutated manually.

    :param scope_number: 0-based number of scope as was listed in scope chain. Only 'local', 'closure' and 'catch' scope types are allowed. Other scopes could be manipulated manually.
    :param variable_name: Variable name.
    :param new_value: New variable value.
    :param call_frame_id: Id of callframe that holds variable.
    """
    params: T_JSON_DICT = dict()
    params["scopeNumber"] = scope_number
    params["variableName"] = variable_name
    params["newValue"] = new_value.to_json()
    params["callFrameId"] = call_frame_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.setVariableValue",
        "params": params,
    }
    json = yield cmd_dict


def step_into(
    break_on_async_call: typing.Optional[bool] = None,
    skip_list: typing.Optional[typing.List[LocationRange]] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Steps into the function call.

    :param break_on_async_call: **(EXPERIMENTAL)** *(Optional)* Debugger will pause on the execution of the first async task which was scheduled before next pause.
    :param skip_list: **(EXPERIMENTAL)** *(Optional)* The skipList specifies location ranges that should be skipped on step into.
    """
    params: T_JSON_DICT = dict()
    if break_on_async_call is not None:
        params["breakOnAsyncCall"] = break_on_async_call
    if skip_list is not None:
        params["skipList"] = [i.to_json() for i in skip_list]
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.stepInto",
        "params": params,
    }
    json = yield cmd_dict


def step_out() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Steps out of the function call.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.stepOut",
    }
    json = yield cmd_dict


def step_over(
    skip_list: typing.Optional[typing.List[LocationRange]] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Steps over the statement.

    :param skip_list: **(EXPERIMENTAL)** *(Optional)* The skipList specifies location ranges that should be skipped on step over.
    """
    params: T_JSON_DICT = dict()
    if skip_list is not None:
        params["skipList"] = [i.to_json() for i in skip_list]
    cmd_dict: T_JSON_DICT = {
        "method": "Debugger.stepOver",
        "params": params,
    }
    json = yield cmd_dict


@deprecated(version="1.3")
@event_class("Debugger.breakpointResolved")
@dataclass
class BreakpointResolved:
    """
    Fired when breakpoint is resolved to an actual script and location.
    Deprecated in favor of ``resolvedBreakpoints`` in the ``scriptParsed`` event.
    """

    #: Breakpoint unique identifier.
    breakpoint_id: BreakpointId
    #: Actual breakpoint location.
    location: Location

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> BreakpointResolved:
        return cls(
            breakpoint_id=BreakpointId.from_json(json["breakpointId"]),
            location=Location.from_json(json["location"]),
        )


@event_class("Debugger.paused")
@dataclass
class Paused:
    """
    Fired when the virtual machine stopped on breakpoint or exception or any other stop criteria.
    """

    #: Call stack the virtual machine stopped on.
    call_frames: typing.List[CallFrame]
    #: Pause reason.
    reason: str
    #: Object containing break-specific auxiliary properties.
    data: typing.Optional[dict]
    #: Hit breakpoints IDs
    hit_breakpoints: typing.Optional[typing.List[str]]
    #: Async stack trace, if any.
    async_stack_trace: typing.Optional[runtime.StackTrace]
    #: Async stack trace, if any.
    async_stack_trace_id: typing.Optional[runtime.StackTraceId]
    #: Never present, will be removed.
    async_call_stack_trace_id: typing.Optional[runtime.StackTraceId]

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> Paused:
        return cls(
            call_frames=[CallFrame.from_json(i) for i in json["callFrames"]],
            reason=str(json["reason"]),
            data=dict(json["data"]) if json.get("data", None) is not None else None,
            hit_breakpoints=[str(i) for i in json["hitBreakpoints"]]
            if json.get("hitBreakpoints", None) is not None
            else None,
            async_stack_trace=runtime.StackTrace.from_json(json["asyncStackTrace"])
            if json.get("asyncStackTrace", None) is not None
            else None,
            async_stack_trace_id=runtime.StackTraceId.from_json(
                json["asyncStackTraceId"]
            )
            if json.get("asyncStackTraceId", None) is not None
            else None,
            async_call_stack_trace_id=runtime.StackTraceId.from_json(
                json["asyncCallStackTraceId"]
            )
            if json.get("asyncCallStackTraceId", None) is not None
            else None,
        )


@event_class("Debugger.resumed")
@dataclass
class Resumed:
    """
    Fired when the virtual machine resumed execution.
    """

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> Resumed:
        return cls()


@event_class("Debugger.scriptFailedToParse")
@dataclass
class ScriptFailedToParse:
    """
    Fired when virtual machine fails to parse the script.
    """

    #: Identifier of the script parsed.
    script_id: runtime.ScriptId
    #: URL or name of the script parsed (if any).
    url: str
    #: Line offset of the script within the resource with given URL (for script tags).
    start_line: int
    #: Column offset of the script within the resource with given URL.
    start_column: int
    #: Last line of the script.
    end_line: int
    #: Length of the last line of the script.
    end_column: int
    #: Specifies script creation context.
    execution_context_id: runtime.ExecutionContextId
    #: Content hash of the script, SHA-256.
    hash_: str
    #: For Wasm modules, the content of the ``build_id`` custom section. For JavaScript the ``debugId`` magic comment.
    build_id: str
    #: Embedder-specific auxiliary data likely matching {isDefault: boolean, type: 'default'``'isolated'``'worker', frameId: string}
    execution_context_aux_data: typing.Optional[dict]
    #: URL of source map associated with script (if any).
    source_map_url: typing.Optional[str]
    #: True, if this script has sourceURL.
    has_source_url: typing.Optional[bool]
    #: True, if this script is ES6 module.
    is_module: typing.Optional[bool]
    #: This script length.
    length: typing.Optional[int]
    #: JavaScript top stack frame of where the script parsed event was triggered if available.
    stack_trace: typing.Optional[runtime.StackTrace]
    #: If the scriptLanguage is WebAssembly, the code section offset in the module.
    code_offset: typing.Optional[int]
    #: The language of the script.
    script_language: typing.Optional[ScriptLanguage]
    #: The name the embedder supplied for this script.
    embedder_name: typing.Optional[str]

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ScriptFailedToParse:
        return cls(
            script_id=runtime.ScriptId.from_json(json["scriptId"]),
            url=str(json["url"]),
            start_line=int(json["startLine"]),
            start_column=int(json["startColumn"]),
            end_line=int(json["endLine"]),
            end_column=int(json["endColumn"]),
            execution_context_id=runtime.ExecutionContextId.from_json(
                json["executionContextId"]
            ),
            hash_=str(json["hash"]),
            build_id=str(json["buildId"]),
            execution_context_aux_data=dict(json["executionContextAuxData"])
            if json.get("executionContextAuxData", None) is not None
            else None,
            source_map_url=str(json["sourceMapURL"])
            if json.get("sourceMapURL", None) is not None
            else None,
            has_source_url=bool(json["hasSourceURL"])
            if json.get("hasSourceURL", None) is not None
            else None,
            is_module=bool(json["isModule"])
            if json.get("isModule", None) is not None
            else None,
            length=int(json["length"])
            if json.get("length", None) is not None
            else None,
            stack_trace=runtime.StackTrace.from_json(json["stackTrace"])
            if json.get("stackTrace", None) is not None
            else None,
            code_offset=int(json["codeOffset"])
            if json.get("codeOffset", None) is not None
            else None,
            script_language=ScriptLanguage.from_json(json["scriptLanguage"])
            if json.get("scriptLanguage", None) is not None
            else None,
            embedder_name=str(json["embedderName"])
            if json.get("embedderName", None) is not None
            else None,
        )


@event_class("Debugger.scriptParsed")
@dataclass
class ScriptParsed:
    """
    Fired when virtual machine parses script. This event is also fired for all known and uncollected
    scripts upon enabling debugger.
    """

    #: Identifier of the script parsed.
    script_id: runtime.ScriptId
    #: URL or name of the script parsed (if any).
    url: str
    #: Line offset of the script within the resource with given URL (for script tags).
    start_line: int
    #: Column offset of the script within the resource with given URL.
    start_column: int
    #: Last line of the script.
    end_line: int
    #: Length of the last line of the script.
    end_column: int
    #: Specifies script creation context.
    execution_context_id: runtime.ExecutionContextId
    #: Content hash of the script, SHA-256.
    hash_: str
    #: For Wasm modules, the content of the ``build_id`` custom section. For JavaScript the ``debugId`` magic comment.
    build_id: str
    #: Embedder-specific auxiliary data likely matching {isDefault: boolean, type: 'default'``'isolated'``'worker', frameId: string}
    execution_context_aux_data: typing.Optional[dict]
    #: True, if this script is generated as a result of the live edit operation.
    is_live_edit: typing.Optional[bool]
    #: URL of source map associated with script (if any).
    source_map_url: typing.Optional[str]
    #: True, if this script has sourceURL.
    has_source_url: typing.Optional[bool]
    #: True, if this script is ES6 module.
    is_module: typing.Optional[bool]
    #: This script length.
    length: typing.Optional[int]
    #: JavaScript top stack frame of where the script parsed event was triggered if available.
    stack_trace: typing.Optional[runtime.StackTrace]
    #: If the scriptLanguage is WebAssembly, the code section offset in the module.
    code_offset: typing.Optional[int]
    #: The language of the script.
    script_language: typing.Optional[ScriptLanguage]
    #: If the scriptLanguage is WebAssembly, the source of debug symbols for the module.
    debug_symbols: typing.Optional[typing.List[DebugSymbols]]
    #: The name the embedder supplied for this script.
    embedder_name: typing.Optional[str]
    #: The list of set breakpoints in this script if calls to ``setBreakpointByUrl``
    #: matches this script's URL or hash. Clients that use this list can ignore the
    #: ``breakpointResolved`` event. They are equivalent.
    resolved_breakpoints: typing.Optional[typing.List[ResolvedBreakpoint]]

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ScriptParsed:
        return cls(
            script_id=runtime.ScriptId.from_json(json["scriptId"]),
            url=str(json["url"]),
            start_line=int(json["startLine"]),
            start_column=int(json["startColumn"]),
            end_line=int(json["endLine"]),
            end_column=int(json["endColumn"]),
            execution_context_id=runtime.ExecutionContextId.from_json(
                json["executionContextId"]
            ),
            hash_=str(json["hash"]),
            build_id=str(json["buildId"]),
            execution_context_aux_data=dict(json["executionContextAuxData"])
            if json.get("executionContextAuxData", None) is not None
            else None,
            is_live_edit=bool(json["isLiveEdit"])
            if json.get("isLiveEdit", None) is not None
            else None,
            source_map_url=str(json["sourceMapURL"])
            if json.get("sourceMapURL", None) is not None
            else None,
            has_source_url=bool(json["hasSourceURL"])
            if json.get("hasSourceURL", None) is not None
            else None,
            is_module=bool(json["isModule"])
            if json.get("isModule", None) is not None
            else None,
            length=int(json["length"])
            if json.get("length", None) is not None
            else None,
            stack_trace=runtime.StackTrace.from_json(json["stackTrace"])
            if json.get("stackTrace", None) is not None
            else None,
            code_offset=int(json["codeOffset"])
            if json.get("codeOffset", None) is not None
            else None,
            script_language=ScriptLanguage.from_json(json["scriptLanguage"])
            if json.get("scriptLanguage", None) is not None
            else None,
            debug_symbols=[DebugSymbols.from_json(i) for i in json["debugSymbols"]]
            if json.get("debugSymbols", None) is not None
            else None,
            embedder_name=str(json["embedderName"])
            if json.get("embedderName", None) is not None
            else None,
            resolved_breakpoints=[
                ResolvedBreakpoint.from_json(i) for i in json["resolvedBreakpoints"]
            ]
            if json.get("resolvedBreakpoints", None) is not None
            else None,
        )



# DO NOT EDIT THIS FILE!
#
# This file is generated from the CDP specification. If you need to make
# changes, edit the generator and regenerate all of the modules.
#
# CDP domain: DOMDebugger

from __future__ import annotations
import enum
import typing
from dataclasses import dataclass
from .util import event_class, T_JSON_DICT

from . import dom
from . import runtime
from deprecated.sphinx import deprecated  # type: ignore


class DOMBreakpointType(enum.Enum):
    """
    DOM breakpoint type.
    """

    SUBTREE_MODIFIED = "subtree-modified"
    ATTRIBUTE_MODIFIED = "attribute-modified"
    NODE_REMOVED = "node-removed"

    def to_json(self) -> str:
        return self.value

    @classmethod
    def from_json(cls, json: str) -> DOMBreakpointType:
        return cls(json)


class CSPViolationType(enum.Enum):
    """
    CSP Violation type.
    """

    TRUSTEDTYPE_SINK_VIOLATION = "trustedtype-sink-violation"
    TRUSTEDTYPE_POLICY_VIOLATION = "trustedtype-policy-violation"

    def to_json(self) -> str:
        return self.value

    @classmethod
    def from_json(cls, json: str) -> CSPViolationType:
        return cls(json)


@dataclass
class EventListener:
    """
    Object event listener.
    """

    #: ``EventListener``'s type.
    type_: str

    #: ``EventListener``'s useCapture.
    use_capture: bool

    #: ``EventListener``'s passive flag.
    passive: bool

    #: ``EventListener``'s once flag.
    once: bool

    #: Script id of the handler code.
    script_id: runtime.ScriptId

    #: Line number in the script (0-based).
    line_number: int

    #: Column number in the script (0-based).
    column_number: int

    #: Event handler function value.
    handler: typing.Optional[runtime.RemoteObject] = None

    #: Event original handler function value.
    original_handler: typing.Optional[runtime.RemoteObject] = None

    #: Node the listener is added to (if any).
    backend_node_id: typing.Optional[dom.BackendNodeId] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["type"] = self.type_
        json["useCapture"] = self.use_capture
        json["passive"] = self.passive
        json["once"] = self.once
        json["scriptId"] = self.script_id.to_json()
        json["lineNumber"] = self.line_number
        json["columnNumber"] = self.column_number
        if self.handler is not None:
            json["handler"] = self.handler.to_json()
        if self.original_handler is not None:
            json["originalHandler"] = self.original_handler.to_json()
        if self.backend_node_id is not None:
            json["backendNodeId"] = self.backend_node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> EventListener:
        return cls(
            type_=str(json["type"]),
            use_capture=bool(json["useCapture"]),
            passive=bool(json["passive"]),
            once=bool(json["once"]),
            script_id=runtime.ScriptId.from_json(json["scriptId"]),
            line_number=int(json["lineNumber"]),
            column_number=int(json["columnNumber"]),
            handler=runtime.RemoteObject.from_json(json["handler"])
            if json.get("handler", None) is not None
            else None,
            original_handler=runtime.RemoteObject.from_json(json["originalHandler"])
            if json.get("originalHandler", None) is not None
            else None,
            backend_node_id=dom.BackendNodeId.from_json(json["backendNodeId"])
            if json.get("backendNodeId", None) is not None
            else None,
        )


def get_event_listeners(
    object_id: runtime.RemoteObjectId,
    depth: typing.Optional[int] = None,
    pierce: typing.Optional[bool] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, typing.List[EventListener]]:
    """
    Returns event listeners of the given object.

    :param object_id: Identifier of the object to return listeners for.
    :param depth: *(Optional)* The maximum depth at which Node children should be retrieved, defaults to 1. Use -1 for the entire subtree or provide an integer larger than 0.
    :param pierce: *(Optional)* Whether or not iframes and shadow roots should be traversed when returning the subtree (default is false). Reports listeners for all contexts if pierce is enabled.
    :returns: Array of relevant listeners.
    """
    params: T_JSON_DICT = dict()
    params["objectId"] = object_id.to_json()
    if depth is not None:
        params["depth"] = depth
    if pierce is not None:
        params["pierce"] = pierce
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.getEventListeners",
        "params": params,
    }
    json = yield cmd_dict
    return [EventListener.from_json(i) for i in json["listeners"]]


def remove_dom_breakpoint(
    node_id: dom.NodeId, type_: DOMBreakpointType
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Removes DOM breakpoint that was set using ``setDOMBreakpoint``.

    :param node_id: Identifier of the node to remove breakpoint from.
    :param type_: Type of the breakpoint to remove.
    """
    params: T_JSON_DICT = dict()
    params["nodeId"] = node_id.to_json()
    params["type"] = type_.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.removeDOMBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


def remove_event_listener_breakpoint(
    event_name: str, target_name: typing.Optional[str] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Removes breakpoint on particular DOM event.

    :param event_name: Event name.
    :param target_name: **(EXPERIMENTAL)** *(Optional)* EventTarget interface name.
    """
    params: T_JSON_DICT = dict()
    params["eventName"] = event_name
    if target_name is not None:
        params["targetName"] = target_name
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.removeEventListenerBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


@deprecated(version="1.3")
def remove_instrumentation_breakpoint(
    event_name: str,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Removes breakpoint on particular native event.

    .. deprecated:: 1.3

    **EXPERIMENTAL**

    :param event_name: Instrumentation name to stop on.
    """
    params: T_JSON_DICT = dict()
    params["eventName"] = event_name
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.removeInstrumentationBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


def remove_xhr_breakpoint(url: str) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Removes breakpoint from XMLHttpRequest.

    :param url: Resource URL substring.
    """
    params: T_JSON_DICT = dict()
    params["url"] = url
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.removeXHRBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


def set_break_on_csp_violation(
    violation_types: typing.List[CSPViolationType],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets breakpoint on particular CSP violations.

    **EXPERIMENTAL**

    :param violation_types: CSP Violations to stop upon.
    """
    params: T_JSON_DICT = dict()
    params["violationTypes"] = [i.to_json() for i in violation_types]
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.setBreakOnCSPViolation",
        "params": params,
    }
    json = yield cmd_dict


def set_dom_breakpoint(
    node_id: dom.NodeId, type_: DOMBreakpointType
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets breakpoint on particular operation with DOM.

    :param node_id: Identifier of the node to set breakpoint on.
    :param type_: Type of the operation to stop upon.
    """
    params: T_JSON_DICT = dict()
    params["nodeId"] = node_id.to_json()
    params["type"] = type_.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.setDOMBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


def set_event_listener_breakpoint(
    event_name: str, target_name: typing.Optional[str] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets breakpoint on particular DOM event.

    :param event_name: DOM Event name to stop on (any DOM event will do).
    :param target_name: **(EXPERIMENTAL)** *(Optional)* EventTarget interface name to stop on. If equal to ```"*"``` or not provided, will stop on any EventTarget.
    """
    params: T_JSON_DICT = dict()
    params["eventName"] = event_name
    if target_name is not None:
        params["targetName"] = target_name
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.setEventListenerBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


@deprecated(version="1.3")
def set_instrumentation_breakpoint(
    event_name: str,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets breakpoint on particular native event.

    .. deprecated:: 1.3

    **EXPERIMENTAL**

    :param event_name: Instrumentation name to stop on.
    """
    params: T_JSON_DICT = dict()
    params["eventName"] = event_name
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.setInstrumentationBreakpoint",
        "params": params,
    }
    json = yield cmd_dict


def set_xhr_breakpoint(url: str) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets breakpoint on XMLHttpRequest.

    :param url: Resource URL substring. All XHRs having this substring in the URL will get stopped upon.
    """
    params: T_JSON_DICT = dict()
    params["url"] = url
    cmd_dict: T_JSON_DICT = {
        "method": "DOMDebugger.setXHRBreakpoint",
        "params": params,
    }
    json = yield cmd_dict



# DO NOT EDIT THIS FILE!
#
# This file is generated from the CDP specification. If you need to make
# changes, edit the generator and regenerate all of the modules.
#
# CDP domain: Overlay (experimental)

from __future__ import annotations
import enum
import typing
from dataclasses import dataclass
from .util import event_class, T_JSON_DICT

from . import dom
from . import page
from . import runtime
from deprecated.sphinx import deprecated  # type: ignore


@dataclass
class SourceOrderConfig:
    """
    Configuration data for drawing the source order of an elements children.
    """

    #: the color to outline the given element in.
    parent_outline_color: dom.RGBA

    #: the color to outline the child elements in.
    child_outline_color: dom.RGBA

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["parentOutlineColor"] = self.parent_outline_color.to_json()
        json["childOutlineColor"] = self.child_outline_color.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> SourceOrderConfig:
        return cls(
            parent_outline_color=dom.RGBA.from_json(json["parentOutlineColor"]),
            child_outline_color=dom.RGBA.from_json(json["childOutlineColor"]),
        )


@dataclass
class GridHighlightConfig:
    """
    Configuration data for the highlighting of Grid elements.
    """

    #: Whether the extension lines from grid cells to the rulers should be shown (default: false).
    show_grid_extension_lines: typing.Optional[bool] = None

    #: Show Positive line number labels (default: false).
    show_positive_line_numbers: typing.Optional[bool] = None

    #: Show Negative line number labels (default: false).
    show_negative_line_numbers: typing.Optional[bool] = None

    #: Show area name labels (default: false).
    show_area_names: typing.Optional[bool] = None

    #: Show line name labels (default: false).
    show_line_names: typing.Optional[bool] = None

    #: Show track size labels (default: false).
    show_track_sizes: typing.Optional[bool] = None

    #: The grid container border highlight color (default: transparent).
    grid_border_color: typing.Optional[dom.RGBA] = None

    #: The cell border color (default: transparent). Deprecated, please use rowLineColor and columnLineColor instead.
    cell_border_color: typing.Optional[dom.RGBA] = None

    #: The row line color (default: transparent).
    row_line_color: typing.Optional[dom.RGBA] = None

    #: The column line color (default: transparent).
    column_line_color: typing.Optional[dom.RGBA] = None

    #: Whether the grid border is dashed (default: false).
    grid_border_dash: typing.Optional[bool] = None

    #: Whether the cell border is dashed (default: false). Deprecated, please us rowLineDash and columnLineDash instead.
    cell_border_dash: typing.Optional[bool] = None

    #: Whether row lines are dashed (default: false).
    row_line_dash: typing.Optional[bool] = None

    #: Whether column lines are dashed (default: false).
    column_line_dash: typing.Optional[bool] = None

    #: The row gap highlight fill color (default: transparent).
    row_gap_color: typing.Optional[dom.RGBA] = None

    #: The row gap hatching fill color (default: transparent).
    row_hatch_color: typing.Optional[dom.RGBA] = None

    #: The column gap highlight fill color (default: transparent).
    column_gap_color: typing.Optional[dom.RGBA] = None

    #: The column gap hatching fill color (default: transparent).
    column_hatch_color: typing.Optional[dom.RGBA] = None

    #: The named grid areas border color (Default: transparent).
    area_border_color: typing.Optional[dom.RGBA] = None

    #: The grid container background color (Default: transparent).
    grid_background_color: typing.Optional[dom.RGBA] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.show_grid_extension_lines is not None:
            json["showGridExtensionLines"] = self.show_grid_extension_lines
        if self.show_positive_line_numbers is not None:
            json["showPositiveLineNumbers"] = self.show_positive_line_numbers
        if self.show_negative_line_numbers is not None:
            json["showNegativeLineNumbers"] = self.show_negative_line_numbers
        if self.show_area_names is not None:
            json["showAreaNames"] = self.show_area_names
        if self.show_line_names is not None:
            json["showLineNames"] = self.show_line_names
        if self.show_track_sizes is not None:
            json["showTrackSizes"] = self.show_track_sizes
        if self.grid_border_color is not None:
            json["gridBorderColor"] = self.grid_border_color.to_json()
        if self.cell_border_color is not None:
            json["cellBorderColor"] = self.cell_border_color.to_json()
        if self.row_line_color is not None:
            json["rowLineColor"] = self.row_line_color.to_json()
        if self.column_line_color is not None:
            json["columnLineColor"] = self.column_line_color.to_json()
        if self.grid_border_dash is not None:
            json["gridBorderDash"] = self.grid_border_dash
        if self.cell_border_dash is not None:
            json["cellBorderDash"] = self.cell_border_dash
        if self.row_line_dash is not None:
            json["rowLineDash"] = self.row_line_dash
        if self.column_line_dash is not None:
            json["columnLineDash"] = self.column_line_dash
        if self.row_gap_color is not None:
            json["rowGapColor"] = self.row_gap_color.to_json()
        if self.row_hatch_color is not None:
            json["rowHatchColor"] = self.row_hatch_color.to_json()
        if self.column_gap_color is not None:
            json["columnGapColor"] = self.column_gap_color.to_json()
        if self.column_hatch_color is not None:
            json["columnHatchColor"] = self.column_hatch_color.to_json()
        if self.area_border_color is not None:
            json["areaBorderColor"] = self.area_border_color.to_json()
        if self.grid_background_color is not None:
            json["gridBackgroundColor"] = self.grid_background_color.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> GridHighlightConfig:
        return cls(
            show_grid_extension_lines=bool(json["showGridExtensionLines"])
            if json.get("showGridExtensionLines", None) is not None
            else None,
            show_positive_line_numbers=bool(json["showPositiveLineNumbers"])
            if json.get("showPositiveLineNumbers", None) is not None
            else None,
            show_negative_line_numbers=bool(json["showNegativeLineNumbers"])
            if json.get("showNegativeLineNumbers", None) is not None
            else None,
            show_area_names=bool(json["showAreaNames"])
            if json.get("showAreaNames", None) is not None
            else None,
            show_line_names=bool(json["showLineNames"])
            if json.get("showLineNames", None) is not None
            else None,
            show_track_sizes=bool(json["showTrackSizes"])
            if json.get("showTrackSizes", None) is not None
            else None,
            grid_border_color=dom.RGBA.from_json(json["gridBorderColor"])
            if json.get("gridBorderColor", None) is not None
            else None,
            cell_border_color=dom.RGBA.from_json(json["cellBorderColor"])
            if json.get("cellBorderColor", None) is not None
            else None,
            row_line_color=dom.RGBA.from_json(json["rowLineColor"])
            if json.get("rowLineColor", None) is not None
            else None,
            column_line_color=dom.RGBA.from_json(json["columnLineColor"])
            if json.get("columnLineColor", None) is not None
            else None,
            grid_border_dash=bool(json["gridBorderDash"])
            if json.get("gridBorderDash", None) is not None
            else None,
            cell_border_dash=bool(json["cellBorderDash"])
            if json.get("cellBorderDash", None) is not None
            else None,
            row_line_dash=bool(json["rowLineDash"])
            if json.get("rowLineDash", None) is not None
            else None,
            column_line_dash=bool(json["columnLineDash"])
            if json.get("columnLineDash", None) is not None
            else None,
            row_gap_color=dom.RGBA.from_json(json["rowGapColor"])
            if json.get("rowGapColor", None) is not None
            else None,
            row_hatch_color=dom.RGBA.from_json(json["rowHatchColor"])
            if json.get("rowHatchColor", None) is not None
            else None,
            column_gap_color=dom.RGBA.from_json(json["columnGapColor"])
            if json.get("columnGapColor", None) is not None
            else None,
            column_hatch_color=dom.RGBA.from_json(json["columnHatchColor"])
            if json.get("columnHatchColor", None) is not None
            else None,
            area_border_color=dom.RGBA.from_json(json["areaBorderColor"])
            if json.get("areaBorderColor", None) is not None
            else None,
            grid_background_color=dom.RGBA.from_json(json["gridBackgroundColor"])
            if json.get("gridBackgroundColor", None) is not None
            else None,
        )


@dataclass
class FlexContainerHighlightConfig:
    """
    Configuration data for the highlighting of Flex container elements.
    """

    #: The style of the container border
    container_border: typing.Optional[LineStyle] = None

    #: The style of the separator between lines
    line_separator: typing.Optional[LineStyle] = None

    #: The style of the separator between items
    item_separator: typing.Optional[LineStyle] = None

    #: Style of content-distribution space on the main axis (justify-content).
    main_distributed_space: typing.Optional[BoxStyle] = None

    #: Style of content-distribution space on the cross axis (align-content).
    cross_distributed_space: typing.Optional[BoxStyle] = None

    #: Style of empty space caused by row gaps (gap/row-gap).
    row_gap_space: typing.Optional[BoxStyle] = None

    #: Style of empty space caused by columns gaps (gap/column-gap).
    column_gap_space: typing.Optional[BoxStyle] = None

    #: Style of the self-alignment line (align-items).
    cross_alignment: typing.Optional[LineStyle] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.container_border is not None:
            json["containerBorder"] = self.container_border.to_json()
        if self.line_separator is not None:
            json["lineSeparator"] = self.line_separator.to_json()
        if self.item_separator is not None:
            json["itemSeparator"] = self.item_separator.to_json()
        if self.main_distributed_space is not None:
            json["mainDistributedSpace"] = self.main_distributed_space.to_json()
        if self.cross_distributed_space is not None:
            json["crossDistributedSpace"] = self.cross_distributed_space.to_json()
        if self.row_gap_space is not None:
            json["rowGapSpace"] = self.row_gap_space.to_json()
        if self.column_gap_space is not None:
            json["columnGapSpace"] = self.column_gap_space.to_json()
        if self.cross_alignment is not None:
            json["crossAlignment"] = self.cross_alignment.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> FlexContainerHighlightConfig:
        return cls(
            container_border=LineStyle.from_json(json["containerBorder"])
            if json.get("containerBorder", None) is not None
            else None,
            line_separator=LineStyle.from_json(json["lineSeparator"])
            if json.get("lineSeparator", None) is not None
            else None,
            item_separator=LineStyle.from_json(json["itemSeparator"])
            if json.get("itemSeparator", None) is not None
            else None,
            main_distributed_space=BoxStyle.from_json(json["mainDistributedSpace"])
            if json.get("mainDistributedSpace", None) is not None
            else None,
            cross_distributed_space=BoxStyle.from_json(json["crossDistributedSpace"])
            if json.get("crossDistributedSpace", None) is not None
            else None,
            row_gap_space=BoxStyle.from_json(json["rowGapSpace"])
            if json.get("rowGapSpace", None) is not None
            else None,
            column_gap_space=BoxStyle.from_json(json["columnGapSpace"])
            if json.get("columnGapSpace", None) is not None
            else None,
            cross_alignment=LineStyle.from_json(json["crossAlignment"])
            if json.get("crossAlignment", None) is not None
            else None,
        )


@dataclass
class FlexItemHighlightConfig:
    """
    Configuration data for the highlighting of Flex item elements.
    """

    #: Style of the box representing the item's base size
    base_size_box: typing.Optional[BoxStyle] = None

    #: Style of the border around the box representing the item's base size
    base_size_border: typing.Optional[LineStyle] = None

    #: Style of the arrow representing if the item grew or shrank
    flexibility_arrow: typing.Optional[LineStyle] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.base_size_box is not None:
            json["baseSizeBox"] = self.base_size_box.to_json()
        if self.base_size_border is not None:
            json["baseSizeBorder"] = self.base_size_border.to_json()
        if self.flexibility_arrow is not None:
            json["flexibilityArrow"] = self.flexibility_arrow.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> FlexItemHighlightConfig:
        return cls(
            base_size_box=BoxStyle.from_json(json["baseSizeBox"])
            if json.get("baseSizeBox", None) is not None
            else None,
            base_size_border=LineStyle.from_json(json["baseSizeBorder"])
            if json.get("baseSizeBorder", None) is not None
            else None,
            flexibility_arrow=LineStyle.from_json(json["flexibilityArrow"])
            if json.get("flexibilityArrow", None) is not None
            else None,
        )


@dataclass
class LineStyle:
    """
    Style information for drawing a line.
    """

    #: The color of the line (default: transparent)
    color: typing.Optional[dom.RGBA] = None

    #: The line pattern (default: solid)
    pattern: typing.Optional[str] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.color is not None:
            json["color"] = self.color.to_json()
        if self.pattern is not None:
            json["pattern"] = self.pattern
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> LineStyle:
        return cls(
            color=dom.RGBA.from_json(json["color"])
            if json.get("color", None) is not None
            else None,
            pattern=str(json["pattern"])
            if json.get("pattern", None) is not None
            else None,
        )


@dataclass
class BoxStyle:
    """
    Style information for drawing a box.
    """

    #: The background color for the box (default: transparent)
    fill_color: typing.Optional[dom.RGBA] = None

    #: The hatching color for the box (default: transparent)
    hatch_color: typing.Optional[dom.RGBA] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.fill_color is not None:
            json["fillColor"] = self.fill_color.to_json()
        if self.hatch_color is not None:
            json["hatchColor"] = self.hatch_color.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> BoxStyle:
        return cls(
            fill_color=dom.RGBA.from_json(json["fillColor"])
            if json.get("fillColor", None) is not None
            else None,
            hatch_color=dom.RGBA.from_json(json["hatchColor"])
            if json.get("hatchColor", None) is not None
            else None,
        )


class ContrastAlgorithm(enum.Enum):
    AA = "aa"
    AAA = "aaa"
    APCA = "apca"

    def to_json(self) -> str:
        return self.value

    @classmethod
    def from_json(cls, json: str) -> ContrastAlgorithm:
        return cls(json)


@dataclass
class HighlightConfig:
    """
    Configuration data for the highlighting of page elements.
    """

    #: Whether the node info tooltip should be shown (default: false).
    show_info: typing.Optional[bool] = None

    #: Whether the node styles in the tooltip (default: false).
    show_styles: typing.Optional[bool] = None

    #: Whether the rulers should be shown (default: false).
    show_rulers: typing.Optional[bool] = None

    #: Whether the a11y info should be shown (default: true).
    show_accessibility_info: typing.Optional[bool] = None

    #: Whether the extension lines from node to the rulers should be shown (default: false).
    show_extension_lines: typing.Optional[bool] = None

    #: The content box highlight fill color (default: transparent).
    content_color: typing.Optional[dom.RGBA] = None

    #: The padding highlight fill color (default: transparent).
    padding_color: typing.Optional[dom.RGBA] = None

    #: The border highlight fill color (default: transparent).
    border_color: typing.Optional[dom.RGBA] = None

    #: The margin highlight fill color (default: transparent).
    margin_color: typing.Optional[dom.RGBA] = None

    #: The event target element highlight fill color (default: transparent).
    event_target_color: typing.Optional[dom.RGBA] = None

    #: The shape outside fill color (default: transparent).
    shape_color: typing.Optional[dom.RGBA] = None

    #: The shape margin fill color (default: transparent).
    shape_margin_color: typing.Optional[dom.RGBA] = None

    #: The grid layout color (default: transparent).
    css_grid_color: typing.Optional[dom.RGBA] = None

    #: The color format used to format color styles (default: hex).
    color_format: typing.Optional[ColorFormat] = None

    #: The grid layout highlight configuration (default: all transparent).
    grid_highlight_config: typing.Optional[GridHighlightConfig] = None

    #: The flex container highlight configuration (default: all transparent).
    flex_container_highlight_config: typing.Optional[FlexContainerHighlightConfig] = (
        None
    )

    #: The flex item highlight configuration (default: all transparent).
    flex_item_highlight_config: typing.Optional[FlexItemHighlightConfig] = None

    #: The contrast algorithm to use for the contrast ratio (default: aa).
    contrast_algorithm: typing.Optional[ContrastAlgorithm] = None

    #: The container query container highlight configuration (default: all transparent).
    container_query_container_highlight_config: typing.Optional[
        ContainerQueryContainerHighlightConfig
    ] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.show_info is not None:
            json["showInfo"] = self.show_info
        if self.show_styles is not None:
            json["showStyles"] = self.show_styles
        if self.show_rulers is not None:
            json["showRulers"] = self.show_rulers
        if self.show_accessibility_info is not None:
            json["showAccessibilityInfo"] = self.show_accessibility_info
        if self.show_extension_lines is not None:
            json["showExtensionLines"] = self.show_extension_lines
        if self.content_color is not None:
            json["contentColor"] = self.content_color.to_json()
        if self.padding_color is not None:
            json["paddingColor"] = self.padding_color.to_json()
        if self.border_color is not None:
            json["borderColor"] = self.border_color.to_json()
        if self.margin_color is not None:
            json["marginColor"] = self.margin_color.to_json()
        if self.event_target_color is not None:
            json["eventTargetColor"] = self.event_target_color.to_json()
        if self.shape_color is not None:
            json["shapeColor"] = self.shape_color.to_json()
        if self.shape_margin_color is not None:
            json["shapeMarginColor"] = self.shape_margin_color.to_json()
        if self.css_grid_color is not None:
            json["cssGridColor"] = self.css_grid_color.to_json()
        if self.color_format is not None:
            json["colorFormat"] = self.color_format.to_json()
        if self.grid_highlight_config is not None:
            json["gridHighlightConfig"] = self.grid_highlight_config.to_json()
        if self.flex_container_highlight_config is not None:
            json["flexContainerHighlightConfig"] = (
                self.flex_container_highlight_config.to_json()
            )
        if self.flex_item_highlight_config is not None:
            json["flexItemHighlightConfig"] = self.flex_item_highlight_config.to_json()
        if self.contrast_algorithm is not None:
            json["contrastAlgorithm"] = self.contrast_algorithm.to_json()
        if self.container_query_container_highlight_config is not None:
            json["containerQueryContainerHighlightConfig"] = (
                self.container_query_container_highlight_config.to_json()
            )
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> HighlightConfig:
        return cls(
            show_info=bool(json["showInfo"])
            if json.get("showInfo", None) is not None
            else None,
            show_styles=bool(json["showStyles"])
            if json.get("showStyles", None) is not None
            else None,
            show_rulers=bool(json["showRulers"])
            if json.get("showRulers", None) is not None
            else None,
            show_accessibility_info=bool(json["showAccessibilityInfo"])
            if json.get("showAccessibilityInfo", None) is not None
            else None,
            show_extension_lines=bool(json["showExtensionLines"])
            if json.get("showExtensionLines", None) is not None
            else None,
            content_color=dom.RGBA.from_json(json["contentColor"])
            if json.get("contentColor", None) is not None
            else None,
            padding_color=dom.RGBA.from_json(json["paddingColor"])
            if json.get("paddingColor", None) is not None
            else None,
            border_color=dom.RGBA.from_json(json["borderColor"])
            if json.get("borderColor", None) is not None
            else None,
            margin_color=dom.RGBA.from_json(json["marginColor"])
            if json.get("marginColor", None) is not None
            else None,
            event_target_color=dom.RGBA.from_json(json["eventTargetColor"])
            if json.get("eventTargetColor", None) is not None
            else None,
            shape_color=dom.RGBA.from_json(json["shapeColor"])
            if json.get("shapeColor", None) is not None
            else None,
            shape_margin_color=dom.RGBA.from_json(json["shapeMarginColor"])
            if json.get("shapeMarginColor", None) is not None
            else None,
            css_grid_color=dom.RGBA.from_json(json["cssGridColor"])
            if json.get("cssGridColor", None) is not None
            else None,
            color_format=ColorFormat.from_json(json["colorFormat"])
            if json.get("colorFormat", None) is not None
            else None,
            grid_highlight_config=GridHighlightConfig.from_json(
                json["gridHighlightConfig"]
            )
            if json.get("gridHighlightConfig", None) is not None
            else None,
            flex_container_highlight_config=FlexContainerHighlightConfig.from_json(
                json["flexContainerHighlightConfig"]
            )
            if json.get("flexContainerHighlightConfig", None) is not None
            else None,
            flex_item_highlight_config=FlexItemHighlightConfig.from_json(
                json["flexItemHighlightConfig"]
            )
            if json.get("flexItemHighlightConfig", None) is not None
            else None,
            contrast_algorithm=ContrastAlgorithm.from_json(json["contrastAlgorithm"])
            if json.get("contrastAlgorithm", None) is not None
            else None,
            container_query_container_highlight_config=ContainerQueryContainerHighlightConfig.from_json(
                json["containerQueryContainerHighlightConfig"]
            )
            if json.get("containerQueryContainerHighlightConfig", None) is not None
            else None,
        )


class ColorFormat(enum.Enum):
    RGB = "rgb"
    HSL = "hsl"
    HWB = "hwb"
    HEX_ = "hex"

    def to_json(self) -> str:
        return self.value

    @classmethod
    def from_json(cls, json: str) -> ColorFormat:
        return cls(json)


@dataclass
class GridNodeHighlightConfig:
    """
    Configurations for Persistent Grid Highlight
    """

    #: A descriptor for the highlight appearance.
    grid_highlight_config: GridHighlightConfig

    #: Identifier of the node to highlight.
    node_id: dom.NodeId

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["gridHighlightConfig"] = self.grid_highlight_config.to_json()
        json["nodeId"] = self.node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> GridNodeHighlightConfig:
        return cls(
            grid_highlight_config=GridHighlightConfig.from_json(
                json["gridHighlightConfig"]
            ),
            node_id=dom.NodeId.from_json(json["nodeId"]),
        )


@dataclass
class FlexNodeHighlightConfig:
    #: A descriptor for the highlight appearance of flex containers.
    flex_container_highlight_config: FlexContainerHighlightConfig

    #: Identifier of the node to highlight.
    node_id: dom.NodeId

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["flexContainerHighlightConfig"] = (
            self.flex_container_highlight_config.to_json()
        )
        json["nodeId"] = self.node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> FlexNodeHighlightConfig:
        return cls(
            flex_container_highlight_config=FlexContainerHighlightConfig.from_json(
                json["flexContainerHighlightConfig"]
            ),
            node_id=dom.NodeId.from_json(json["nodeId"]),
        )


@dataclass
class ScrollSnapContainerHighlightConfig:
    #: The style of the snapport border (default: transparent)
    snapport_border: typing.Optional[LineStyle] = None

    #: The style of the snap area border (default: transparent)
    snap_area_border: typing.Optional[LineStyle] = None

    #: The margin highlight fill color (default: transparent).
    scroll_margin_color: typing.Optional[dom.RGBA] = None

    #: The padding highlight fill color (default: transparent).
    scroll_padding_color: typing.Optional[dom.RGBA] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.snapport_border is not None:
            json["snapportBorder"] = self.snapport_border.to_json()
        if self.snap_area_border is not None:
            json["snapAreaBorder"] = self.snap_area_border.to_json()
        if self.scroll_margin_color is not None:
            json["scrollMarginColor"] = self.scroll_margin_color.to_json()
        if self.scroll_padding_color is not None:
            json["scrollPaddingColor"] = self.scroll_padding_color.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ScrollSnapContainerHighlightConfig:
        return cls(
            snapport_border=LineStyle.from_json(json["snapportBorder"])
            if json.get("snapportBorder", None) is not None
            else None,
            snap_area_border=LineStyle.from_json(json["snapAreaBorder"])
            if json.get("snapAreaBorder", None) is not None
            else None,
            scroll_margin_color=dom.RGBA.from_json(json["scrollMarginColor"])
            if json.get("scrollMarginColor", None) is not None
            else None,
            scroll_padding_color=dom.RGBA.from_json(json["scrollPaddingColor"])
            if json.get("scrollPaddingColor", None) is not None
            else None,
        )


@dataclass
class ScrollSnapHighlightConfig:
    #: A descriptor for the highlight appearance of scroll snap containers.
    scroll_snap_container_highlight_config: ScrollSnapContainerHighlightConfig

    #: Identifier of the node to highlight.
    node_id: dom.NodeId

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["scrollSnapContainerHighlightConfig"] = (
            self.scroll_snap_container_highlight_config.to_json()
        )
        json["nodeId"] = self.node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ScrollSnapHighlightConfig:
        return cls(
            scroll_snap_container_highlight_config=ScrollSnapContainerHighlightConfig.from_json(
                json["scrollSnapContainerHighlightConfig"]
            ),
            node_id=dom.NodeId.from_json(json["nodeId"]),
        )


@dataclass
class HingeConfig:
    """
    Configuration for dual screen hinge
    """

    #: A rectangle represent hinge
    rect: dom.Rect

    #: The content box highlight fill color (default: a dark color).
    content_color: typing.Optional[dom.RGBA] = None

    #: The content box highlight outline color (default: transparent).
    outline_color: typing.Optional[dom.RGBA] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["rect"] = self.rect.to_json()
        if self.content_color is not None:
            json["contentColor"] = self.content_color.to_json()
        if self.outline_color is not None:
            json["outlineColor"] = self.outline_color.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> HingeConfig:
        return cls(
            rect=dom.Rect.from_json(json["rect"]),
            content_color=dom.RGBA.from_json(json["contentColor"])
            if json.get("contentColor", None) is not None
            else None,
            outline_color=dom.RGBA.from_json(json["outlineColor"])
            if json.get("outlineColor", None) is not None
            else None,
        )


@dataclass
class WindowControlsOverlayConfig:
    """
    Configuration for Window Controls Overlay
    """

    #: Whether the title bar CSS should be shown when emulating the Window Controls Overlay.
    show_css: bool

    #: Selected platforms to show the overlay.
    selected_platform: str

    #: The theme color defined in app manifest.
    theme_color: str

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["showCSS"] = self.show_css
        json["selectedPlatform"] = self.selected_platform
        json["themeColor"] = self.theme_color
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> WindowControlsOverlayConfig:
        return cls(
            show_css=bool(json["showCSS"]),
            selected_platform=str(json["selectedPlatform"]),
            theme_color=str(json["themeColor"]),
        )


@dataclass
class ContainerQueryHighlightConfig:
    #: A descriptor for the highlight appearance of container query containers.
    container_query_container_highlight_config: ContainerQueryContainerHighlightConfig

    #: Identifier of the container node to highlight.
    node_id: dom.NodeId

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["containerQueryContainerHighlightConfig"] = (
            self.container_query_container_highlight_config.to_json()
        )
        json["nodeId"] = self.node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ContainerQueryHighlightConfig:
        return cls(
            container_query_container_highlight_config=ContainerQueryContainerHighlightConfig.from_json(
                json["containerQueryContainerHighlightConfig"]
            ),
            node_id=dom.NodeId.from_json(json["nodeId"]),
        )


@dataclass
class ContainerQueryContainerHighlightConfig:
    #: The style of the container border.
    container_border: typing.Optional[LineStyle] = None

    #: The style of the descendants' borders.
    descendant_border: typing.Optional[LineStyle] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.container_border is not None:
            json["containerBorder"] = self.container_border.to_json()
        if self.descendant_border is not None:
            json["descendantBorder"] = self.descendant_border.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ContainerQueryContainerHighlightConfig:
        return cls(
            container_border=LineStyle.from_json(json["containerBorder"])
            if json.get("containerBorder", None) is not None
            else None,
            descendant_border=LineStyle.from_json(json["descendantBorder"])
            if json.get("descendantBorder", None) is not None
            else None,
        )


@dataclass
class IsolatedElementHighlightConfig:
    #: A descriptor for the highlight appearance of an element in isolation mode.
    isolation_mode_highlight_config: IsolationModeHighlightConfig

    #: Identifier of the isolated element to highlight.
    node_id: dom.NodeId

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["isolationModeHighlightConfig"] = (
            self.isolation_mode_highlight_config.to_json()
        )
        json["nodeId"] = self.node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> IsolatedElementHighlightConfig:
        return cls(
            isolation_mode_highlight_config=IsolationModeHighlightConfig.from_json(
                json["isolationModeHighlightConfig"]
            ),
            node_id=dom.NodeId.from_json(json["nodeId"]),
        )


@dataclass
class IsolationModeHighlightConfig:
    #: The fill color of the resizers (default: transparent).
    resizer_color: typing.Optional[dom.RGBA] = None

    #: The fill color for resizer handles (default: transparent).
    resizer_handle_color: typing.Optional[dom.RGBA] = None

    #: The fill color for the mask covering non-isolated elements (default: transparent).
    mask_color: typing.Optional[dom.RGBA] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        if self.resizer_color is not None:
            json["resizerColor"] = self.resizer_color.to_json()
        if self.resizer_handle_color is not None:
            json["resizerHandleColor"] = self.resizer_handle_color.to_json()
        if self.mask_color is not None:
            json["maskColor"] = self.mask_color.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> IsolationModeHighlightConfig:
        return cls(
            resizer_color=dom.RGBA.from_json(json["resizerColor"])
            if json.get("resizerColor", None) is not None
            else None,
            resizer_handle_color=dom.RGBA.from_json(json["resizerHandleColor"])
            if json.get("resizerHandleColor", None) is not None
            else None,
            mask_color=dom.RGBA.from_json(json["maskColor"])
            if json.get("maskColor", None) is not None
            else None,
        )


class InspectMode(enum.Enum):
    SEARCH_FOR_NODE = "searchForNode"
    SEARCH_FOR_UA_SHADOW_DOM = "searchForUAShadowDOM"
    CAPTURE_AREA_SCREENSHOT = "captureAreaScreenshot"
    SHOW_DISTANCES = "showDistances"
    NONE = "none"

    def to_json(self) -> str:
        return self.value

    @classmethod
    def from_json(cls, json: str) -> InspectMode:
        return cls(json)


def disable() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Disables domain notifications.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.disable",
    }
    json = yield cmd_dict


def enable() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Enables domain notifications.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.enable",
    }
    json = yield cmd_dict


def get_highlight_object_for_test(
    node_id: dom.NodeId,
    include_distance: typing.Optional[bool] = None,
    include_style: typing.Optional[bool] = None,
    color_format: typing.Optional[ColorFormat] = None,
    show_accessibility_info: typing.Optional[bool] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, dict]:
    """
    For testing.

    :param node_id: Id of the node to get highlight object for.
    :param include_distance: *(Optional)* Whether to include distance info.
    :param include_style: *(Optional)* Whether to include style info.
    :param color_format: *(Optional)* The color format to get config with (default: hex).
    :param show_accessibility_info: *(Optional)* Whether to show accessibility info (default: true).
    :returns: Highlight data for the node.
    """
    params: T_JSON_DICT = dict()
    params["nodeId"] = node_id.to_json()
    if include_distance is not None:
        params["includeDistance"] = include_distance
    if include_style is not None:
        params["includeStyle"] = include_style
    if color_format is not None:
        params["colorFormat"] = color_format.to_json()
    if show_accessibility_info is not None:
        params["showAccessibilityInfo"] = show_accessibility_info
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.getHighlightObjectForTest",
        "params": params,
    }
    json = yield cmd_dict
    return dict(json["highlight"])


def get_grid_highlight_objects_for_test(
    node_ids: typing.List[dom.NodeId],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, dict]:
    """
    For Persistent Grid testing.

    :param node_ids: Ids of the node to get highlight object for.
    :returns: Grid Highlight data for the node ids provided.
    """
    params: T_JSON_DICT = dict()
    params["nodeIds"] = [i.to_json() for i in node_ids]
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.getGridHighlightObjectsForTest",
        "params": params,
    }
    json = yield cmd_dict
    return dict(json["highlights"])


def get_source_order_highlight_object_for_test(
    node_id: dom.NodeId,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, dict]:
    """
    For Source Order Viewer testing.

    :param node_id: Id of the node to highlight.
    :returns: Source order highlight data for the node id provided.
    """
    params: T_JSON_DICT = dict()
    params["nodeId"] = node_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.getSourceOrderHighlightObjectForTest",
        "params": params,
    }
    json = yield cmd_dict
    return dict(json["highlight"])


def hide_highlight() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Hides any highlight.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.hideHighlight",
    }
    json = yield cmd_dict


@deprecated(version="1.3")
def highlight_frame(
    frame_id: page.FrameId,
    content_color: typing.Optional[dom.RGBA] = None,
    content_outline_color: typing.Optional[dom.RGBA] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlights owner element of the frame with given id.
    Deprecated: Doesn't work reliably and cannot be fixed due to process
    separation (the owner node might be in a different process). Determine
    the owner node in the client and use highlightNode.

    .. deprecated:: 1.3

    :param frame_id: Identifier of the frame to highlight.
    :param content_color: *(Optional)* The content box highlight fill color (default: transparent).
    :param content_outline_color: *(Optional)* The content box highlight outline color (default: transparent).
    """
    params: T_JSON_DICT = dict()
    params["frameId"] = frame_id.to_json()
    if content_color is not None:
        params["contentColor"] = content_color.to_json()
    if content_outline_color is not None:
        params["contentOutlineColor"] = content_outline_color.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.highlightFrame",
        "params": params,
    }
    json = yield cmd_dict


def highlight_node(
    highlight_config: HighlightConfig,
    node_id: typing.Optional[dom.NodeId] = None,
    backend_node_id: typing.Optional[dom.BackendNodeId] = None,
    object_id: typing.Optional[runtime.RemoteObjectId] = None,
    selector: typing.Optional[str] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlights DOM node with given id or with the given JavaScript object wrapper. Either nodeId or
    objectId must be specified.

    :param highlight_config: A descriptor for the highlight appearance.
    :param node_id: *(Optional)* Identifier of the node to highlight.
    :param backend_node_id: *(Optional)* Identifier of the backend node to highlight.
    :param object_id: *(Optional)* JavaScript object id of the node to be highlighted.
    :param selector: *(Optional)* Selectors to highlight relevant nodes.
    """
    params: T_JSON_DICT = dict()
    params["highlightConfig"] = highlight_config.to_json()
    if node_id is not None:
        params["nodeId"] = node_id.to_json()
    if backend_node_id is not None:
        params["backendNodeId"] = backend_node_id.to_json()
    if object_id is not None:
        params["objectId"] = object_id.to_json()
    if selector is not None:
        params["selector"] = selector
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.highlightNode",
        "params": params,
    }
    json = yield cmd_dict


def highlight_quad(
    quad: dom.Quad,
    color: typing.Optional[dom.RGBA] = None,
    outline_color: typing.Optional[dom.RGBA] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlights given quad. Coordinates are absolute with respect to the main frame viewport.

    :param quad: Quad to highlight
    :param color: *(Optional)* The highlight fill color (default: transparent).
    :param outline_color: *(Optional)* The highlight outline color (default: transparent).
    """
    params: T_JSON_DICT = dict()
    params["quad"] = quad.to_json()
    if color is not None:
        params["color"] = color.to_json()
    if outline_color is not None:
        params["outlineColor"] = outline_color.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.highlightQuad",
        "params": params,
    }
    json = yield cmd_dict


def highlight_rect(
    x: int,
    y: int,
    width: int,
    height: int,
    color: typing.Optional[dom.RGBA] = None,
    outline_color: typing.Optional[dom.RGBA] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlights given rectangle. Coordinates are absolute with respect to the main frame viewport.

    :param x: X coordinate
    :param y: Y coordinate
    :param width: Rectangle width
    :param height: Rectangle height
    :param color: *(Optional)* The highlight fill color (default: transparent).
    :param outline_color: *(Optional)* The highlight outline color (default: transparent).
    """
    params: T_JSON_DICT = dict()
    params["x"] = x
    params["y"] = y
    params["width"] = width
    params["height"] = height
    if color is not None:
        params["color"] = color.to_json()
    if outline_color is not None:
        params["outlineColor"] = outline_color.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.highlightRect",
        "params": params,
    }
    json = yield cmd_dict


def highlight_source_order(
    source_order_config: SourceOrderConfig,
    node_id: typing.Optional[dom.NodeId] = None,
    backend_node_id: typing.Optional[dom.BackendNodeId] = None,
    object_id: typing.Optional[runtime.RemoteObjectId] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlights the source order of the children of the DOM node with given id or with the given
    JavaScript object wrapper. Either nodeId or objectId must be specified.

    :param source_order_config: A descriptor for the appearance of the overlay drawing.
    :param node_id: *(Optional)* Identifier of the node to highlight.
    :param backend_node_id: *(Optional)* Identifier of the backend node to highlight.
    :param object_id: *(Optional)* JavaScript object id of the node to be highlighted.
    """
    params: T_JSON_DICT = dict()
    params["sourceOrderConfig"] = source_order_config.to_json()
    if node_id is not None:
        params["nodeId"] = node_id.to_json()
    if backend_node_id is not None:
        params["backendNodeId"] = backend_node_id.to_json()
    if object_id is not None:
        params["objectId"] = object_id.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.highlightSourceOrder",
        "params": params,
    }
    json = yield cmd_dict


def set_inspect_mode(
    mode: InspectMode, highlight_config: typing.Optional[HighlightConfig] = None
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Enters the 'inspect' mode. In this mode, elements that user is hovering over are highlighted.
    Backend then generates 'inspectNodeRequested' event upon element selection.

    :param mode: Set an inspection mode.
    :param highlight_config: *(Optional)* A descriptor for the highlight appearance of hovered-over nodes. May be omitted if ```enabled == false```.
    """
    params: T_JSON_DICT = dict()
    params["mode"] = mode.to_json()
    if highlight_config is not None:
        params["highlightConfig"] = highlight_config.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setInspectMode",
        "params": params,
    }
    json = yield cmd_dict


def set_show_ad_highlights(
    show: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlights owner element of all frames detected to be ads.

    :param show: True for showing ad highlights
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowAdHighlights",
        "params": params,
    }
    json = yield cmd_dict


def set_paused_in_debugger_message(
    message: typing.Optional[str] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    :param message: *(Optional)* The message to display, also triggers resume and step over controls.
    """
    params: T_JSON_DICT = dict()
    if message is not None:
        params["message"] = message
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setPausedInDebuggerMessage",
        "params": params,
    }
    json = yield cmd_dict


def set_show_debug_borders(
    show: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Requests that backend shows debug borders on layers

    :param show: True for showing debug borders
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowDebugBorders",
        "params": params,
    }
    json = yield cmd_dict


def set_show_fps_counter(
    show: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Requests that backend shows the FPS counter

    :param show: True for showing the FPS counter
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowFPSCounter",
        "params": params,
    }
    json = yield cmd_dict


def set_show_grid_overlays(
    grid_node_highlight_configs: typing.List[GridNodeHighlightConfig],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Highlight multiple elements with the CSS Grid overlay.

    :param grid_node_highlight_configs: An array of node identifiers and descriptors for the highlight appearance.
    """
    params: T_JSON_DICT = dict()
    params["gridNodeHighlightConfigs"] = [
        i.to_json() for i in grid_node_highlight_configs
    ]
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowGridOverlays",
        "params": params,
    }
    json = yield cmd_dict


def set_show_flex_overlays(
    flex_node_highlight_configs: typing.List[FlexNodeHighlightConfig],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    :param flex_node_highlight_configs: An array of node identifiers and descriptors for the highlight appearance.
    """
    params: T_JSON_DICT = dict()
    params["flexNodeHighlightConfigs"] = [
        i.to_json() for i in flex_node_highlight_configs
    ]
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowFlexOverlays",
        "params": params,
    }
    json = yield cmd_dict


def set_show_scroll_snap_overlays(
    scroll_snap_highlight_configs: typing.List[ScrollSnapHighlightConfig],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    :param scroll_snap_highlight_configs: An array of node identifiers and descriptors for the highlight appearance.
    """
    params: T_JSON_DICT = dict()
    params["scrollSnapHighlightConfigs"] = [
        i.to_json() for i in scroll_snap_highlight_configs
    ]
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowScrollSnapOverlays",
        "params": params,
    }
    json = yield cmd_dict


def set_show_container_query_overlays(
    container_query_highlight_configs: typing.List[ContainerQueryHighlightConfig],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    :param container_query_highlight_configs: An array of node identifiers and descriptors for the highlight appearance.
    """
    params: T_JSON_DICT = dict()
    params["containerQueryHighlightConfigs"] = [
        i.to_json() for i in container_query_highlight_configs
    ]
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowContainerQueryOverlays",
        "params": params,
    }
    json = yield cmd_dict


def set_show_paint_rects(
    result: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Requests that backend shows paint rectangles

    :param result: True for showing paint rectangles
    """
    params: T_JSON_DICT = dict()
    params["result"] = result
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowPaintRects",
        "params": params,
    }
    json = yield cmd_dict


def set_show_layout_shift_regions(
    result: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Requests that backend shows layout shift regions

    :param result: True for showing layout shift regions
    """
    params: T_JSON_DICT = dict()
    params["result"] = result
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowLayoutShiftRegions",
        "params": params,
    }
    json = yield cmd_dict


def set_show_scroll_bottleneck_rects(
    show: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Requests that backend shows scroll bottleneck rects

    :param show: True for showing scroll bottleneck rects
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowScrollBottleneckRects",
        "params": params,
    }
    json = yield cmd_dict


@deprecated(version="1.3")
def set_show_hit_test_borders(
    show: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Deprecated, no longer has any effect.

    .. deprecated:: 1.3

    :param show: True for showing hit-test borders
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowHitTestBorders",
        "params": params,
    }
    json = yield cmd_dict


@deprecated(version="1.3")
def set_show_web_vitals(show: bool) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Deprecated, no longer has any effect.

    .. deprecated:: 1.3

    :param show:
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowWebVitals",
        "params": params,
    }
    json = yield cmd_dict


def set_show_viewport_size_on_resize(
    show: bool,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Paints viewport size upon main frame resize.

    :param show: Whether to paint size or not.
    """
    params: T_JSON_DICT = dict()
    params["show"] = show
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowViewportSizeOnResize",
        "params": params,
    }
    json = yield cmd_dict


def set_show_hinge(
    hinge_config: typing.Optional[HingeConfig] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Add a dual screen device hinge

    :param hinge_config: *(Optional)* hinge data, null means hideHinge
    """
    params: T_JSON_DICT = dict()
    if hinge_config is not None:
        params["hingeConfig"] = hinge_config.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowHinge",
        "params": params,
    }
    json = yield cmd_dict


def set_show_isolated_elements(
    isolated_element_highlight_configs: typing.List[IsolatedElementHighlightConfig],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Show elements in isolation mode with overlays.

    :param isolated_element_highlight_configs: An array of node identifiers and descriptors for the highlight appearance.
    """
    params: T_JSON_DICT = dict()
    params["isolatedElementHighlightConfigs"] = [
        i.to_json() for i in isolated_element_highlight_configs
    ]
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowIsolatedElements",
        "params": params,
    }
    json = yield cmd_dict


def set_show_window_controls_overlay(
    window_controls_overlay_config: typing.Optional[WindowControlsOverlayConfig] = None,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Show Window Controls Overlay for PWA

    :param window_controls_overlay_config: *(Optional)* Window Controls Overlay data, null means hide Window Controls Overlay
    """
    params: T_JSON_DICT = dict()
    if window_controls_overlay_config is not None:
        params["windowControlsOverlayConfig"] = window_controls_overlay_config.to_json()
    cmd_dict: T_JSON_DICT = {
        "method": "Overlay.setShowWindowControlsOverlay",
        "params": params,
    }
    json = yield cmd_dict


@event_class("Overlay.inspectNodeRequested")
@dataclass
class InspectNodeRequested:
    """
    Fired when the node should be inspected. This happens after call to ``setInspectMode`` or when
    user manually inspects an element.
    """

    #: Id of the node to inspect.
    backend_node_id: dom.BackendNodeId

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> InspectNodeRequested:
        return cls(backend_node_id=dom.BackendNodeId.from_json(json["backendNodeId"]))


@event_class("Overlay.nodeHighlightRequested")
@dataclass
class NodeHighlightRequested:
    """
    Fired when the node should be highlighted. This happens after call to ``setInspectMode``.
    """

    node_id: dom.NodeId

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> NodeHighlightRequested:
        return cls(node_id=dom.NodeId.from_json(json["nodeId"]))


@event_class("Overlay.screenshotRequested")
@dataclass
class ScreenshotRequested:
    """
    Fired when user asks to capture screenshot of some area on the page.
    """

    #: Viewport to capture, in device independent pixels (dip).
    viewport: page.Viewport

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ScreenshotRequested:
        return cls(viewport=page.Viewport.from_json(json["viewport"]))


@event_class("Overlay.inspectModeCanceled")
@dataclass
class InspectModeCanceled:
    """
    Fired when user cancels the inspect mode.
    """

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> InspectModeCanceled:
        return cls()



import asyncio

import zendriver as zd
from zendriver import Element, Tab


async def wait_for_cards(page: Tab, initial_card_count: int) -> list[Element]:
    while True:
        card_container = await page.select("#card-container")
        cards = card_container.children
        if len(cards) > initial_card_count:
            return cards
        await asyncio.sleep(0.5)


async def main() -> None:
    browser = await zd.start()
    page = await browser.get(
        "https://slensky.com/zendriver-examples/scrollable-cards.html",
    )

    # Wait for cards to load
    cards = await wait_for_cards(page, initial_card_count=0)

    # Now we can print the cards
    # (shows first 10 cards: Card 1, Card 2...Card 9, Card 10)
    for card in cards:
        print(card.text)

    await browser.stop()


if __name__ == "__main__":
    asyncio.run(main())



import importlib.util
import sys
from pathlib import Path
from types import ModuleType


def import_from_path(file_path: Path | str) -> ModuleType:
    if isinstance(file_path, str):
        file_path = Path(file_path)

    module_name = file_path.stem
    spec = importlib.util.spec_from_file_location(module_name, file_path)
    if spec is None or spec.loader is None:
        raise ImportError(f"Cannot find module {module_name}")

    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    return module



import asyncio
import logging
import os
import signal
import sys
from contextlib import AbstractAsyncContextManager
from enum import Enum
from threading import Event
from typing import AsyncGenerator

import pytest

import zendriver as zd

logger = logging.getLogger(__name__)


class BrowserMode(Enum):
    HEADLESS = "headless"
    HEADFUL = "headful"
    ALL = "all"

    @property
    def fixture_params(self):
        if self == BrowserMode.HEADLESS:
            return [{"headless": True}]
        elif self == BrowserMode.HEADFUL:
            return [{"headless": False}]
        elif self == BrowserMode.ALL:
            return [{"headless": True}, {"headless": False}]


NEXT_TEST_EVENT = Event()


class TestConfig:
    BROWSER_MODE = BrowserMode(os.getenv("ZENDRIVER_TEST_BROWSERS", "all"))
    PAUSE_AFTER_TEST = os.getenv("ZENDRIVER_PAUSE_AFTER_TEST", "false") == "true"
    SANDBOX = os.getenv("ZENDRIVER_TEST_SANDBOX", "false") == "true"
    USE_WAYLAND = os.getenv("WAYLAND_DISPLAY") is not None


class CreateBrowser(AbstractAsyncContextManager):
    def __init__(
        self,
        *,
        headless: bool = True,
        sandbox: bool = TestConfig.SANDBOX,
        browser_args: list[str] | None = None,
        browser_connection_max_tries: int = 15,
        browser_connection_timeout: float = 3.0,
    ):
        args = []
        if not headless and TestConfig.USE_WAYLAND:
            # use wayland backend instead of x11
            args.extend(
                ["--disable-features=UseOzonePlatform", "--ozone-platform=wayland"]
            )
        if browser_args is not None:
            args.extend(browser_args)

        self.config = zd.Config(
            headless=headless,
            sandbox=sandbox,
            browser_args=args,
            browser_connection_max_tries=browser_connection_max_tries,
            browser_connection_timeout=browser_connection_timeout,
        )

        self.browser: zd.Browser | None = None
        self.browser_pid: int | None = None

    async def __aenter__(self) -> zd.Browser:
        self.browser = await zd.start(self.config)
        browser_pid = self.browser._process_pid
        assert browser_pid is not None and browser_pid > 0
        await self.browser.wait(0)
        return self.browser

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.browser is not None:
            await self.browser.stop()
            assert self.browser_pid is None


@pytest.fixture
def create_browser() -> type[CreateBrowser]:
    if sys.platform == "win32":
        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())  # type: ignore

    return CreateBrowser


@pytest.fixture(params=TestConfig.BROWSER_MODE.fixture_params)
def headless(request: pytest.FixtureRequest) -> bool:
    return request.param["headless"]


@pytest.fixture
async def browser(
    headless: bool, create_browser: type[CreateBrowser]
) -> AsyncGenerator[zd.Browser, None]:
    NEXT_TEST_EVENT.clear()

    async with create_browser(headless=headless) as browser:
        yield browser

    if TestConfig.PAUSE_AFTER_TEST:
        logger.info(
            "Pausing after test. Send next test hotkey (default Mod+Return) to continue to next test"
        )
        NEXT_TEST_EVENT.wait()
    await browser.stop()
    assert browser._process_pid is None


# signal handler for starting next test
def handle_next_test(signum, frame):
    if not TestConfig.PAUSE_AFTER_TEST:
        logger.warning(
            "Next test signal received, but ZENDRIVER_PAUSE_AFTER_TEST is not set."
        )
        logger.warning(
            "To enable pausing after each test, set ZENDRIVER_PAUSE_AFTER_TEST=true"
        )
        return

    NEXT_TEST_EVENT.set()


if hasattr(signal, "SIGUSR1"):
    signal.signal(signal.SIGUSR1, handle_next_test)
else:
    logger.warning(
        "SIGUSR1 not available on this platform, handle_next_test will not be called."
    )



# DO NOT EDIT THIS FILE!
#
# This file is generated from the CDP specification. If you need to make
# changes, edit the generator and regenerate all of the modules.
#
# CDP domain: Animation (experimental)

from __future__ import annotations
import enum
import typing
from dataclasses import dataclass
from .util import event_class, T_JSON_DICT

from . import dom
from . import runtime


@dataclass
class Animation:
    """
    Animation instance.
    """

    #: ``Animation``'s id.
    id_: str

    #: ``Animation``'s name.
    name: str

    #: ``Animation``'s internal paused state.
    paused_state: bool

    #: ``Animation``'s play state.
    play_state: str

    #: ``Animation``'s playback rate.
    playback_rate: float

    #: ``Animation``'s start time.
    #: Milliseconds for time based animations and
    #: percentage [0 - 100] for scroll driven animations
    #: (i.e. when viewOrScrollTimeline exists).
    start_time: float

    #: ``Animation``'s current time.
    current_time: float

    #: Animation type of ``Animation``.
    type_: str

    #: ``Animation``'s source animation node.
    source: typing.Optional[AnimationEffect] = None

    #: A unique ID for ``Animation`` representing the sources that triggered this CSS
    #: animation/transition.
    css_id: typing.Optional[str] = None

    #: View or scroll timeline
    view_or_scroll_timeline: typing.Optional[ViewOrScrollTimeline] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["id"] = self.id_
        json["name"] = self.name
        json["pausedState"] = self.paused_state
        json["playState"] = self.play_state
        json["playbackRate"] = self.playback_rate
        json["startTime"] = self.start_time
        json["currentTime"] = self.current_time
        json["type"] = self.type_
        if self.source is not None:
            json["source"] = self.source.to_json()
        if self.css_id is not None:
            json["cssId"] = self.css_id
        if self.view_or_scroll_timeline is not None:
            json["viewOrScrollTimeline"] = self.view_or_scroll_timeline.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> Animation:
        return cls(
            id_=str(json["id"]),
            name=str(json["name"]),
            paused_state=bool(json["pausedState"]),
            play_state=str(json["playState"]),
            playback_rate=float(json["playbackRate"]),
            start_time=float(json["startTime"]),
            current_time=float(json["currentTime"]),
            type_=str(json["type"]),
            source=AnimationEffect.from_json(json["source"])
            if json.get("source", None) is not None
            else None,
            css_id=str(json["cssId"]) if json.get("cssId", None) is not None else None,
            view_or_scroll_timeline=ViewOrScrollTimeline.from_json(
                json["viewOrScrollTimeline"]
            )
            if json.get("viewOrScrollTimeline", None) is not None
            else None,
        )


@dataclass
class ViewOrScrollTimeline:
    """
    Timeline instance
    """

    #: Orientation of the scroll
    axis: dom.ScrollOrientation

    #: Scroll container node
    source_node_id: typing.Optional[dom.BackendNodeId] = None

    #: Represents the starting scroll position of the timeline
    #: as a length offset in pixels from scroll origin.
    start_offset: typing.Optional[float] = None

    #: Represents the ending scroll position of the timeline
    #: as a length offset in pixels from scroll origin.
    end_offset: typing.Optional[float] = None

    #: The element whose principal box's visibility in the
    #: scrollport defined the progress of the timeline.
    #: Does not exist for animations with ScrollTimeline
    subject_node_id: typing.Optional[dom.BackendNodeId] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["axis"] = self.axis.to_json()
        if self.source_node_id is not None:
            json["sourceNodeId"] = self.source_node_id.to_json()
        if self.start_offset is not None:
            json["startOffset"] = self.start_offset
        if self.end_offset is not None:
            json["endOffset"] = self.end_offset
        if self.subject_node_id is not None:
            json["subjectNodeId"] = self.subject_node_id.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> ViewOrScrollTimeline:
        return cls(
            axis=dom.ScrollOrientation.from_json(json["axis"]),
            source_node_id=dom.BackendNodeId.from_json(json["sourceNodeId"])
            if json.get("sourceNodeId", None) is not None
            else None,
            start_offset=float(json["startOffset"])
            if json.get("startOffset", None) is not None
            else None,
            end_offset=float(json["endOffset"])
            if json.get("endOffset", None) is not None
            else None,
            subject_node_id=dom.BackendNodeId.from_json(json["subjectNodeId"])
            if json.get("subjectNodeId", None) is not None
            else None,
        )


@dataclass
class AnimationEffect:
    """
    AnimationEffect instance
    """

    #: ``AnimationEffect``'s delay.
    delay: float

    #: ``AnimationEffect``'s end delay.
    end_delay: float

    #: ``AnimationEffect``'s iteration start.
    iteration_start: float

    #: ``AnimationEffect``'s iterations.
    iterations: float

    #: ``AnimationEffect``'s iteration duration.
    #: Milliseconds for time based animations and
    #: percentage [0 - 100] for scroll driven animations
    #: (i.e. when viewOrScrollTimeline exists).
    duration: float

    #: ``AnimationEffect``'s playback direction.
    direction: str

    #: ``AnimationEffect``'s fill mode.
    fill: str

    #: ``AnimationEffect``'s timing function.
    easing: str

    #: ``AnimationEffect``'s target node.
    backend_node_id: typing.Optional[dom.BackendNodeId] = None

    #: ``AnimationEffect``'s keyframes.
    keyframes_rule: typing.Optional[KeyframesRule] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["delay"] = self.delay
        json["endDelay"] = self.end_delay
        json["iterationStart"] = self.iteration_start
        json["iterations"] = self.iterations
        json["duration"] = self.duration
        json["direction"] = self.direction
        json["fill"] = self.fill
        json["easing"] = self.easing
        if self.backend_node_id is not None:
            json["backendNodeId"] = self.backend_node_id.to_json()
        if self.keyframes_rule is not None:
            json["keyframesRule"] = self.keyframes_rule.to_json()
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> AnimationEffect:
        return cls(
            delay=float(json["delay"]),
            end_delay=float(json["endDelay"]),
            iteration_start=float(json["iterationStart"]),
            iterations=float(json["iterations"]),
            duration=float(json["duration"]),
            direction=str(json["direction"]),
            fill=str(json["fill"]),
            easing=str(json["easing"]),
            backend_node_id=dom.BackendNodeId.from_json(json["backendNodeId"])
            if json.get("backendNodeId", None) is not None
            else None,
            keyframes_rule=KeyframesRule.from_json(json["keyframesRule"])
            if json.get("keyframesRule", None) is not None
            else None,
        )


@dataclass
class KeyframesRule:
    """
    Keyframes Rule
    """

    #: List of animation keyframes.
    keyframes: typing.List[KeyframeStyle]

    #: CSS keyframed animation's name.
    name: typing.Optional[str] = None

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["keyframes"] = [i.to_json() for i in self.keyframes]
        if self.name is not None:
            json["name"] = self.name
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> KeyframesRule:
        return cls(
            keyframes=[KeyframeStyle.from_json(i) for i in json["keyframes"]],
            name=str(json["name"]) if json.get("name", None) is not None else None,
        )


@dataclass
class KeyframeStyle:
    """
    Keyframe Style
    """

    #: Keyframe's time offset.
    offset: str

    #: ``AnimationEffect``'s timing function.
    easing: str

    def to_json(self) -> T_JSON_DICT:
        json: T_JSON_DICT = dict()
        json["offset"] = self.offset
        json["easing"] = self.easing
        return json

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> KeyframeStyle:
        return cls(
            offset=str(json["offset"]),
            easing=str(json["easing"]),
        )


def disable() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Disables animation domain notifications.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.disable",
    }
    json = yield cmd_dict


def enable() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Enables animation domain notifications.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.enable",
    }
    json = yield cmd_dict


def get_current_time(id_: str) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, float]:
    """
    Returns the current time of the an animation.

    :param id_: Id of animation.
    :returns: Current time of the page.
    """
    params: T_JSON_DICT = dict()
    params["id"] = id_
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.getCurrentTime",
        "params": params,
    }
    json = yield cmd_dict
    return float(json["currentTime"])


def get_playback_rate() -> typing.Generator[T_JSON_DICT, T_JSON_DICT, float]:
    """
    Gets the playback rate of the document timeline.

    :returns: Playback rate for animations on page.
    """
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.getPlaybackRate",
    }
    json = yield cmd_dict
    return float(json["playbackRate"])


def release_animations(
    animations: typing.List[str],
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Releases a set of animations to no longer be manipulated.

    :param animations: List of animation ids to seek.
    """
    params: T_JSON_DICT = dict()
    params["animations"] = [i for i in animations]
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.releaseAnimations",
        "params": params,
    }
    json = yield cmd_dict


def resolve_animation(
    animation_id: str,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, runtime.RemoteObject]:
    """
    Gets the remote object of the Animation.

    :param animation_id: Animation id.
    :returns: Corresponding remote object.
    """
    params: T_JSON_DICT = dict()
    params["animationId"] = animation_id
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.resolveAnimation",
        "params": params,
    }
    json = yield cmd_dict
    return runtime.RemoteObject.from_json(json["remoteObject"])


def seek_animations(
    animations: typing.List[str], current_time: float
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Seek a set of animations to a particular time within each animation.

    :param animations: List of animation ids to seek.
    :param current_time: Set the current time of each animation.
    """
    params: T_JSON_DICT = dict()
    params["animations"] = [i for i in animations]
    params["currentTime"] = current_time
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.seekAnimations",
        "params": params,
    }
    json = yield cmd_dict


def set_paused(
    animations: typing.List[str], paused: bool
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets the paused state of a set of animations.

    :param animations: Animations to set the pause state of.
    :param paused: Paused state to set to.
    """
    params: T_JSON_DICT = dict()
    params["animations"] = [i for i in animations]
    params["paused"] = paused
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.setPaused",
        "params": params,
    }
    json = yield cmd_dict


def set_playback_rate(
    playback_rate: float,
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets the playback rate of the document timeline.

    :param playback_rate: Playback rate for animations on page
    """
    params: T_JSON_DICT = dict()
    params["playbackRate"] = playback_rate
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.setPlaybackRate",
        "params": params,
    }
    json = yield cmd_dict


def set_timing(
    animation_id: str, duration: float, delay: float
) -> typing.Generator[T_JSON_DICT, T_JSON_DICT, None]:
    """
    Sets the timing of an animation node.

    :param animation_id: Animation id.
    :param duration: Duration of the animation.
    :param delay: Delay of the animation.
    """
    params: T_JSON_DICT = dict()
    params["animationId"] = animation_id
    params["duration"] = duration
    params["delay"] = delay
    cmd_dict: T_JSON_DICT = {
        "method": "Animation.setTiming",
        "params": params,
    }
    json = yield cmd_dict


@event_class("Animation.animationCanceled")
@dataclass
class AnimationCanceled:
    """
    Event for when an animation has been cancelled.
    """

    #: Id of the animation that was cancelled.
    id_: str

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> AnimationCanceled:
        return cls(id_=str(json["id"]))


@event_class("Animation.animationCreated")
@dataclass
class AnimationCreated:
    """
    Event for each animation that has been created.
    """

    #: Id of the animation that was created.
    id_: str

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> AnimationCreated:
        return cls(id_=str(json["id"]))


@event_class("Animation.animationStarted")
@dataclass
class AnimationStarted:
    """
    Event for animation that has been started.
    """

    #: Animation that was started.
    animation: Animation

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> AnimationStarted:
        return cls(animation=Animation.from_json(json["animation"]))


@event_class("Animation.animationUpdated")
@dataclass
class AnimationUpdated:
    """
    Event for animation that has been updated.
    """

    #: Animation that was updated.
    animation: Animation

    @classmethod
    def from_json(cls, json: T_JSON_DICT) -> AnimationUpdated:
        return cls(animation=Animation.from_json(json["animation"]))
